from __future__ import annotations

from tcsoa.gen.Internal.MultiSite._2007_06.ObjectDirectory import DistributedAppProperties, QueryItemIdRecordResponse, DistributedAppResponse, DescribePublicationRecordsResponse, ClientInfoProperties, LocatePublishedObjectResponse, PublicationRecordProperties, GetODSVersionResponse, ItemIdRecordsProperties, QueryPublicationRecordsResponse
from tcsoa.gen.Internal.MultiSite._2007_06.TcEntObjectDirectory import QueryPublicationRecordsResponse, DescribePublicationRecordsResponse, TcEntClientInfoProperties, TcEntPublicationRecordProperties, RepublishObjectInputProperties, QueryPublicationRecordsProperties, GetODSVersionResponse
from tcsoa.gen.Internal.MultiSite._2007_06.RemoteOperation import IDSM7DistributedAppRes, IDSM6DescribeItemRes, IDSM6StartSendObjectsRes, IDSM7AskVersionRes, IDSM4XferExportFileIn, IDSM6StartSendObjectsIn, IDSM1RepublishObjectIn, IDSM2DescribeObjectIn, IDSM6StartExportItemIdListIn, IDSM6PerformSendObjectsRes, IDSM7AskVersionIn, IDSM4CheckSyncStatusIn, IDSM4SetSyncOptionsIn, IDSM4XferImportFileIn, IDSM7StartExportRes, IDSM1StartAskInfoRes, IDSM3EndImportIn, IDSM4XferImportFileRes, IDSM7StartExportIn, IDSM1StartAskInfoIn, IDSM1EndImportRes, IDSM6StartExportItemIdListRes, IDSM2DescribeObjectRes, IDSM6PerformSendObjectsIn, IDSM4GetErrorStackRes, IDSM1EndExportIn, IDSM6EndSendObjectsIn, IDSM4CheckSyncStatusRes, IDSM6EndExportItemIdListIn, IDSM1LocateObjectIn, IDSM4RemoteNotificationRes, IDSM9DescribeObjectRes, IDSM2VersionCheckRes, IDSM4SetSyncOptionsRes, IDSM1VerifyObjectsIn, IDSM3StartImportRes, IDSM1VerifyObjectsRes, IDSM3ExportStatusIn, IDSM7DistributedAppIn, IDSM1XferInfoFileRes, IDSM4RemoteNotificationIn, IDSM4XferExportFileRes, IDSM6DescribeItemIn, IDSM1LocateObjectRes, IDSM3StartImportIn, IDSM1XferInfoFileIn, IDSM1EndAskInfoIn, IDSM4GetErrorStackIn, IDSM3ExportStatusRes, IDSM9DescribeObjectIn, IDSM2VersionCheckIn
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class RemoteOperationService(TcService):

    @classmethod
    def idsm1EndAskInfoSvc(cls, inputs: IDSM1EndAskInfoIn) -> ServiceData:
        """
        The 'idsm1EndAskInfoSvc' operation is used to finish the info file transfer.  This is the last operation in the
        sequence of transferring info file and finishes the info file transfer process.
        
        This operation is integrated with Multi-Site Remote Procedure Call (RPC) framework and is callable only in
        Hypertext Transfer Protocol (HTTP) based Multi-Site environment. This operation acts as a wrapper call to
        server side RPC function and hence is invoked from within the Multi-Site framework. Consequently no other
        clients can invoke this operation.
        
        The corresponding RPC call is' idsm1_end_ask_info_1_svc().
        '
        
        Use cases:
        This operation is called while creating or updating the dataset. 'idsm1StartAskInfoSvc' call will be made to
        get the schema info of the given site, then type and classinfo will be fetched out of it, on successful
        completion of the call, info file is transferred to server staging directory using 'idsm1XferInfoFileSvc'. The
        'idsm1EndAskInfoSvc' closes the communication channel and finishes the file transfer.
        """
        return cls.execute_soa_method(
            method_name='idsm1EndAskInfoSvc',
            library='Internal-MultiSite',
            service_date='2007_06',
            service_name='RemoteOperation',
            params={'inputs': inputs},
            response_cls=ServiceData,
        )

    @classmethod
    def idsm1EndExportSvc(cls, inputs: IDSM1EndExportIn) -> ServiceData:
        """
        The 'idsm1EndExportSvc' operation completes the Remote Import transaction. This is the last operation in the
        sequence and cleans up files from Multi-Site staging directory and cannot be called in isolation. 
        
        This operation is integrated with Multi-Site Remote Procedure Call (RPC) framework and is callable only in
        Hypertext Transfer Protocol (HTTP) based Multi-Site environment. This operation acts as a wrapper call to
        server side RPC function and hence is invoked from within the Multi-Site framework. Consequently no other
        clients can invoke this operation.
        
        The corresponding RPC call is 'idsm1_end_export_1_svc().'
        
        
        Use cases:
        This operation is called while creating or updating the dataset. 'idsm1StartAskInfoSvc' call will be made to
        get the schema info of the given site, then type and classinfo will be fetched out of it, on successful
        completion of the call, info file is transferred to server staging directory using 'idsm1XferInfoFileSvc'. The
        'idsm1EndAskInfoSvc' closes the communication channel and finishes the file transfer.
        """
        return cls.execute_soa_method(
            method_name='idsm1EndExportSvc',
            library='Internal-MultiSite',
            service_date='2007_06',
            service_name='RemoteOperation',
            params={'inputs': inputs},
            response_cls=ServiceData,
        )

    @classmethod
    def idsm1LocateObjectSvc(cls, inputs: IDSM1LocateObjectIn) -> IDSM1LocateObjectRes:
        """
        The 'idsm1LocateObjectSvc' operation locates the owning site of the remote object given in
        'IDSM1LocateObjectIn' input structure.
        
        This operation is integrated with Multi-Site Remote Procedure Call (RPC) framework and is callable only in HTTP
        based Multi-Site environment. This operation acts as a wrapper call to server side RPC function and hence is
        invoked from within the Multi-Site framework. Consequently no other clients can invoke this operation.
        
        Corresponding RPC call is' idsm1_locate_object_1_svc().'
        
        
        Use cases:
        'idsm1LocateObjectSvc' operation invoked during remote import of stub object. Export Assembly from site1 to
        site2 with including BOM. On site2, open assembly in PSE (Product Structure Editor) and it shows components as
        Remote Objects, do import on that object(s), which tries to find the owning site of the object. It tries to
        locate the stub object by finding its owning site by checking publication record for that object by calling
        'locatePublishedObjects'.
        """
        return cls.execute_soa_method(
            method_name='idsm1LocateObjectSvc',
            library='Internal-MultiSite',
            service_date='2007_06',
            service_name='RemoteOperation',
            params={'inputs': inputs},
            response_cls=IDSM1LocateObjectRes,
        )

    @classmethod
    def idsm1RepublishObjectSvc(cls, inputs: IDSM1RepublishObjectIn) -> ServiceData:
        """
        publish the object to ODS
        """
        return cls.execute_soa_method(
            method_name='idsm1RepublishObjectSvc',
            library='Internal-MultiSite',
            service_date='2007_06',
            service_name='RemoteOperation',
            params={'inputs': inputs},
            response_cls=ServiceData,
        )

    @classmethod
    def idsm1StartAskInfoSvc(cls, inputs: IDSM1StartAskInfoIn) -> IDSM1StartAskInfoRes:
        """
        The 'idsm1StartAskInfoSvc' operation is call used to get the information about the server side staging
        directory i.e. where the files are being transferred on the server side while get type-class mapping file from
        remote site. This basically opens a file transfer channel used to transfer files from the server.
        
        This is the first operation in the sequence which gets information about the Multi-Site staging directory and
        cannot be called in isolation. 
        
        This operation is integrated with Multi-Site Remote Procedure Call (RPC) framework and is callable only in HTTP
        based Multi-Site environment. This operation acts as a wrapper call to server side RPC function and hence is
        invoked from within the Multi-Site framework. Consequently no other clients can invoke this operation.
        
        Corresponding RPC call is 'idsm1_start_ask_info_1_svc().
        '
        
        Use cases:
        'idsm1StartAskInfoSvc' operation is invoked while getting type class files from the remote site in HTTP
        multisite environment. This is called from database_verify() utility.
        Operation sequence is as below:
        - idsm1StartAskInfoSvc
        - idsm1XferInfoFileSvc
        
        """
        return cls.execute_soa_method(
            method_name='idsm1StartAskInfoSvc',
            library='Internal-MultiSite',
            service_date='2007_06',
            service_name='RemoteOperation',
            params={'inputs': inputs},
            response_cls=IDSM1StartAskInfoRes,
        )

    @classmethod
    def idsm1VerifyObjectsSvc(cls, input: IDSM1VerifyObjectsIn) -> IDSM1VerifyObjectsRes:
        """
        The 'idsm1VerifyObjectsSvc' operation is used to verify if the object exist at the target site given in
        IDSM1VerifyObjectsIn input object.
        
        This operation is integrated with Multi-Site Remote Procedure Call (RPC) framework and is callable only in HTTP
        based Multi-Site environment. This operation acts as a wrapper call to server side RPC function and hence is
        invoked from within the Multi-Site framework. Consequently no other clients can invoke this operation.
        
        Corresponding RPC call is' idsm1_verify_objects_1_svc().'
        
        
        
        Use cases:
        'idsm1VerifyObjectsSvc' operation is called in case of importing stub objects from structure manager where it
        tries to check whether these objects exist at target site and their verdict in HTTP multisite environment.
        """
        return cls.execute_soa_method(
            method_name='idsm1VerifyObjectsSvc',
            library='Internal-MultiSite',
            service_date='2007_06',
            service_name='RemoteOperation',
            params={'input': input},
            response_cls=IDSM1VerifyObjectsRes,
        )

    @classmethod
    def idsm1XferInfoFileSvc(cls, inputs: IDSM1XferInfoFileIn) -> IDSM1XferInfoFileRes:
        """
        The 'idsm1XferInfoFileSvc' operation transfers the type class mapping file from the remote site(s) Multi-Site
        staging directory to the local site Multi-Site staging directory. 
        
        This is the second operation in the sequence and cannot be called in isolation. 
        
        This operation is integrated with Multi-Site Remote Procedure Call (RPC) framework and is callable only in HTTP
        based Multi-Site environment. This operation acts as a wrapper call to server side RPC function and hence is
        invoked from within the Multi-Site framework. Consequently no other clients can invoke this operation.
        
        Corresponding RPC call is' idsm1_xfer_info_file_1_svc().'
        
        
        Use cases:
        idsm1XferInfoFileSvc operation is invoked while getting type class files from the remote site in HTTP multisite
        environment. This is called from database_verify() utility.
        Operation sequence is as below:
        - idsm1StartAskInfoSvc
        - idsm1XferInfoFileSvc
        
        """
        return cls.execute_soa_method(
            method_name='idsm1XferInfoFileSvc',
            library='Internal-MultiSite',
            service_date='2007_06',
            service_name='RemoteOperation',
            params={'inputs': inputs},
            response_cls=IDSM1XferInfoFileRes,
        )

    @classmethod
    def idsm2DescribeObjectSvc(cls, inputs: IDSM2DescribeObjectIn) -> IDSM2DescribeObjectRes:
        """
        The 'idsm2DescribeObjectSvc' operation used to retrieve the object description from IDSM server.  Consumer
        should provide the client information and the tag of the object, as an input.
        
        This operation is integrated with Multi-Site Remote Procedure Call (RPC) framework and is callable only in
        Hypertext Transfer Protocol (HTTP) based Multi-Site environment. This operation acts as a wrapper call to
        server side RPC function and hence is invoked from within the Multi-Site framework. Consequently no other
        clients can invoke this operation.
        
        The corresponding RPC call is' idsm2_describe_object_1_svc().'
        
        
        Use cases:
        To import the remote objects at the local site, object(s) published to the Object Directory Service (ODS) needs
        to be first searched and then imported. While loading the ODS published remote objects, this operation is
        called to get the object information.
        """
        return cls.execute_soa_method(
            method_name='idsm2DescribeObjectSvc',
            library='Internal-MultiSite',
            service_date='2007_06',
            service_name='RemoteOperation',
            params={'inputs': inputs},
            response_cls=IDSM2DescribeObjectRes,
        )

    @classmethod
    def idsm2VersionCheckSvc(cls, inputs: IDSM2VersionCheckIn) -> IDSM2VersionCheckRes:
        """
        The 'idsm2VersionCheckSvc' operation gets the Integrated Distributed Services Manager (IDSM) server version
        
        This operation is integrated with Multi-Site Remote Procedure Call (RPC) framework and is callable only in HTTP
        based Multi-Site environment. This operation acts as a wrapper call to server side RPC function and hence is
        invoked from within the Multi-Site framework. Consequently no other clients can invoke this operation.
        
        Corresponding RPC call is 'idsm2_version_check_1_svc().'
        
        
        Use cases:
        'idsm2VersionCheckSvc' operation is called prior to all IDSM requests made by client to check version of IDSM
        for compatibility in HTTP multisite environment.  It is also used by some of the Distributed(DIST) ITKs prior
        to make any IDSM request.
        """
        return cls.execute_soa_method(
            method_name='idsm2VersionCheckSvc',
            library='Internal-MultiSite',
            service_date='2007_06',
            service_name='RemoteOperation',
            params={'inputs': inputs},
            response_cls=IDSM2VersionCheckRes,
        )

    @classmethod
    def idsm3EndImportSvc(cls, inputs: IDSM3EndImportIn) -> IDSM1EndImportRes:
        """
        The 'idsm3EndImportSvc' operation imports remote business objects to the local site. When a user at Source Site
        selects business objects (e.g, Item) for remote export, these objects are written into object.meta file at the
        Source Site and transferred over to the Target Site. Integrated Distributed Services Manager (IDSM) service at
        the Target Site imports the content of this file and idsm3EndImportSvc is the last operation in the sequence
        and cannot be called in isolation.
        
        This operation is integrated with Multi-Site Remote Procedure Call (RPC) framework and is callable only in
        Hypertext Transfer Protocol (HTTP) based Multi-Site environment. This operation acts as a wrapper call to
        server side RPC function and hence is invoked from within the Multi-Site framework. Consequently no other
        clients can invoke this operation.
        
        The corresponding RPC call is 'idsm3_end_import_1_svc().'
        
        
        Use cases:
        To Remote Export business object(s), a series of operations needs to be performed and 'idsm3EndImportSvc' gets
        invoked to complete the remote import transaction. 
        Operation sequence for remote export
        - idsm3StartImportSvc
        - idsm4XferImportFileSvc
        - idsm3EndImportSvc
        
        """
        return cls.execute_soa_method(
            method_name='idsm3EndImportSvc',
            library='Internal-MultiSite',
            service_date='2007_06',
            service_name='RemoteOperation',
            params={'inputs': inputs},
            response_cls=IDSM1EndImportRes,
        )

    @classmethod
    def idsm3ExportStatusSvc(cls, inputs: IDSM3ExportStatusIn) -> IDSM3ExportStatusRes:
        """
        The 'idsm3ExportStatusSvc' operation returns the export/import status of business objects.
        
        This operation is integrated with Multi-Site Remote Procedure Call (RPC) framework and is callable only in
        Hypertext Transfer Protocol (HTTP) based Multi-Site environment. This operation acts as a wrapper call to
        server side RPC function and hence is invoked from within the Multi-Site framework. Consequently no other
        clients can invoke this operation.
        
        The corresponding rpc call is 'idsm3_export_status_1_svc().'
        
        
        Use cases:
        - In case of the remote import, this operation is used to check the status of the remote import invoked by the
        earlier call 'idsm3StartExportSvc'(). The status informs the remote site whether import is complete and if it s
        in process, then how many workspace objects are exported so far. The reported workspace objects will be in the
        multiples of 'checkClientAfterNWsos' supplied in the 'IDSM3ExportStatusIn' structure.
        - In case of the remote export, this operation is used to check the status of the remote export finished by the
        earlier call 'idsm3EndImportSvc'(). The status informs the remote site whether export is complete and if it s
        in process, then how many workspace objects are exported so far. The reported workspace objects will be in the
        multiples of 'checkClientAfterNWsos' supplied in the 'IDSM3ExportStatusIn' structure.
        
        """
        return cls.execute_soa_method(
            method_name='idsm3ExportStatusSvc',
            library='Internal-MultiSite',
            service_date='2007_06',
            service_name='RemoteOperation',
            params={'inputs': inputs},
            response_cls=IDSM3ExportStatusRes,
        )

    @classmethod
    def idsm3StartImportSvc(cls, inputs: IDSM3StartImportIn) -> IDSM3StartImportRes:
        """
        The 'idsm3StartImportSvc' operation gets information about the server side staging directory i.e. where the
        files are being transferred on the server during export.
        
        This is the first operation in the sequence and cannot be called in isolation. 
        
        This operation is integrated with Multi-Site Remote Procedure Call (RPC) framework and is callable only in HTTP
        based Multi-Site environment. This operation acts as a wrapper call to server side RPC function and hence is
        invoked from within the Multi-Site framework. Consequently no other clients can invoke this operation.
        
        Corresponding RPC call is 'idsm3_start_import_1_svc().'
        
        
        Use cases:
        To Remote Export business object(s), a series of operations needs to be performed and Idsm3StartImportSvc gets
        invoked as a first step. 
        Operation sequence for remote export.
        - idsm3StartImportSvc
        - idsm4XferImportFileSvc
        - idsm3EndImportSvc
        
        """
        return cls.execute_soa_method(
            method_name='idsm3StartImportSvc',
            library='Internal-MultiSite',
            service_date='2007_06',
            service_name='RemoteOperation',
            params={'inputs': inputs},
            response_cls=IDSM3StartImportRes,
        )

    @classmethod
    def idsm4CheckSyncStatusSvc(cls, inputs: IDSM4CheckSyncStatusIn) -> IDSM4CheckSyncStatusRes:
        """
        The 'idsm4CheckSyncStatusSvc' operation is used to check the sync status of the input set of object(s) in the
        IDSM4CheckSyncStatusIn structure. 
        
        This operation is integrated with Multi-Site Remote Procedure Call (RPC) framework and is callable only in
        Hypertext Transfer Protocol (HTTP) based Multi-Site environment. This operation acts as a wrapper call to
        server side RPC function and hence is invoked from within the Multi-Site framework. Consequently no other
        clients can invoke this operation.
        
        The corresponding RPC call is 'idsm4_check_sync_status_1_svc().
        '
        
        
        Use cases:
        This operation is called from the auto sync handler to retrieve the object sync state. The auto sync handler
        used to extend and customize task actions.
        """
        return cls.execute_soa_method(
            method_name='idsm4CheckSyncStatusSvc',
            library='Internal-MultiSite',
            service_date='2007_06',
            service_name='RemoteOperation',
            params={'inputs': inputs},
            response_cls=IDSM4CheckSyncStatusRes,
        )

    @classmethod
    def idsm4GetErrorStackSvc(cls, inputs: IDSM4GetErrorStackIn) -> IDSM4GetErrorStackRes:
        """
        The 'idsm4GetErrorStackSvc' operation is used to retrieve the error stack.
        
        This operation is integrated with Multi-Site Remote Procedure Call (RPC) framework and is callable only in
        Hypertext Transfer Protocol (HTTP) based Multi-Site environment. This operation acts as a wrapper call to
        server side RPC function and hence is invoked from within the Multi-Site framework. Consequently no other
        clients can invoke this operation.
        
        The corresponding rpc call is' idsm4_get_error_stack_1_svc().'
        
        
        Use cases:
        While performing the checkpoint transaction, this operation is called from the data_share & data_sync utility.
        """
        return cls.execute_soa_method(
            method_name='idsm4GetErrorStackSvc',
            library='Internal-MultiSite',
            service_date='2007_06',
            service_name='RemoteOperation',
            params={'inputs': inputs},
            response_cls=IDSM4GetErrorStackRes,
        )

    @classmethod
    def idsm4RemoteNotificationSvc(cls, inputs: IDSM4RemoteNotificationIn) -> IDSM4RemoteNotificationRes:
        """
        The 'idsm4RemoteNotificationSvc' operation performs remote notification and subscription for object at remote
        site based on IDSM4RemoteNotificationIn input structure.
        
        This is the last operation in the sequence and cannot be called in isolation. 
        
        This operation is integrated with Multi-Site Remote Procedure Call (RPC) framework and is callable only in HTTP
        based Multi-Site environment. This operation acts as a wrapper call to server side RPC function and hence is
        invoked from within the Multi-Site framework. Consequently no other clients can invoke this operation.
        
        Corresponding RPC call is 'idsm4_remote_notification_1_svc().'
        
        
        Use cases:
        'idsm4RemoteNotificationSvc' operation is called in automatic synchronization to notify and subscribe remote
        site in HTTP multisite environment.
        """
        return cls.execute_soa_method(
            method_name='idsm4RemoteNotificationSvc',
            library='Internal-MultiSite',
            service_date='2007_06',
            service_name='RemoteOperation',
            params={'inputs': inputs},
            response_cls=IDSM4RemoteNotificationRes,
        )

    @classmethod
    def idsm4SetSyncOptionsSvc(cls, inputs: IDSM4SetSyncOptionsIn) -> IDSM4SetSyncOptionsRes:
        """
        The 'idsm4SetSyncOptionsSvc' operation used to sets sync options on the objects after object import from
        importing site.
        
        This operation is integrated with Multi-Site Remote Procedure Call (RPC) framework and is callable only in HTTP
        based Multi-Site environment. This operation acts as a wrapper call to server side RPC function and hence is
        invoked from within the Multi-Site framework. Consequently no other clients can invoke this operation.
        
        Corresponding RPC call is 'idsm4_set_sync_options_1_svc ().'
        
        
        Use cases:
        'idsm4SetSyncOptionsSvc' operation is called during remote import and is called from importing site in HTTP
        multisite environment.
        """
        return cls.execute_soa_method(
            method_name='idsm4SetSyncOptionsSvc',
            library='Internal-MultiSite',
            service_date='2007_06',
            service_name='RemoteOperation',
            params={'inputs': inputs},
            response_cls=IDSM4SetSyncOptionsRes,
        )

    @classmethod
    def idsm4XferExportFileSvc(cls, inputs: IDSM4XferExportFileIn) -> IDSM4XferExportFileRes:
        """
        The 'idsm4XferExportFileSvc' operation transfers object.meta file and bulk data files from the local site s
        Multi-Site staging directory to the remote Integrated Distributed Services Manager (IDSM) site s Multi-Site
        staging directory.
        
        This operation is integrated with Multi-Site Remote Procedure Call (RPC) framework and is callable only in
        Hypertext Transfer Protocol (HTTP) based Multi-Site environment. This operation acts as a wrapper call to
        server side RPC function and hence is invoked from within the Multi-Site framework. Consequently no other
        clients can invoke this operation.
        
        The corresponding RPC call is 'idsm4_xfer_export_file_1_svc().'
        
        
        Use cases:
        To remote import the business object, first the source site user creates an Item and publishes it to the
        default Object Directory Service (ODS).  The target site user performs remote search with the item id and
        imports it to the target site using the remote import operation. On successful completion of remote import
        operation, the item is imported to the target site database. To Remote Import the business object, the series
        of operations needs to be performed and this operation gets invoked as a last step in the Remote Import
        transaction. The sequence of operations are as below 
        - idsm7StartExportSvc
        - idsm3ExportStatusSvc
        - idsm4XferExportFileSvc
        - idsm1EndExportSvc
        
        """
        return cls.execute_soa_method(
            method_name='idsm4XferExportFileSvc',
            library='Internal-MultiSite',
            service_date='2007_06',
            service_name='RemoteOperation',
            params={'inputs': inputs},
            response_cls=IDSM4XferExportFileRes,
        )

    @classmethod
    def idsm4XferImportFileSvc(cls, inputs: IDSM4XferImportFileIn) -> IDSM4XferImportFileRes:
        """
        The 'idsm4XferImportFileSvc' operation transfers object.meta file and bulk data files from the remote site(s)
        Multi-Site staging directory to the local site Multi-Site staging directory. 
        
        This operation is integrated with Multi-Site Remote Procedure Call (RPC) framework and is callable only in
        Hypertext Transfer Protocol (HTTP) based Multi-Site environment. This operation acts as a wrapper call to
        server side RPC function and hence is invoked from within the Multi-Site framework. Consequently no other
        clients can invoke this operation.
        
        The corresponding RPC call is 'idsm4_xfer_import_file_1_svc().'
        
        
        Use cases:
        To Remote Export business object(s), a series of operations needs to be performed and idsm4XferImportFileSv
        gets invoked to perform the object.meta file and bulk data file from remote site to local site. 
        Operation sequence for remote export
        - idsm3StartImportSvc
        - idsm4XferImportFileSvc
        - idsm3EndImportSvc
        
        """
        return cls.execute_soa_method(
            method_name='idsm4XferImportFileSvc',
            library='Internal-MultiSite',
            service_date='2007_06',
            service_name='RemoteOperation',
            params={'inputs': inputs},
            response_cls=IDSM4XferImportFileRes,
        )

    @classmethod
    def idsm6DescribeItemSvc(cls, inputs: IDSM6DescribeItemIn) -> IDSM6DescribeItemRes:
        """
        The 'idsm6DescribeItemSvc' operation retrieves item information for a given item.
        
        This operation is integrated with Multi-Site Remote Procedure Call (RPC) framework and is callable only in
        Hypertext Transfer Protocol (HTTP) based Multi-Site environment. This operation acts as a wrapper call to
        server side RPC function and hence is invoked from within the Multi-Site framework. Consequently no other
        clients can invoke this operation.
        
        The corresponding RPC call is 'idsm6_describe_item_1_svc().'
        
        
        Use cases:
        This operation is called from data_share utility, for reporting the duplicate items. For the supplied item id,
        this operation will get item information available with the remote Integrated Distributed Services Manager
        (IDSM) server and reports the duplicate items.
        """
        return cls.execute_soa_method(
            method_name='idsm6DescribeItemSvc',
            library='Internal-MultiSite',
            service_date='2007_06',
            service_name='RemoteOperation',
            params={'inputs': inputs},
            response_cls=IDSM6DescribeItemRes,
        )

    @classmethod
    def idsm6EndExportItemIdListSvc(cls, inputs: IDSM6EndExportItemIdListIn) -> ServiceData:
        """
        The 'idsm6EndExportItemIdListSvc' is used to delete the server staging directory, and set finished status. 
        This is the last operation in the sequence of remote item id import operation and finishes the remote item id
        import process.
        
        This operation is integrated with Multi-Site Remote Procedure Call (RPC) framework and is callable only in
        Hypertext Transfer Protocol (HTTP) based Multi-Site environment. This operation acts as a wrapper call to
        server side RPC function and hence is invoked from within the Multi-Site framework. Consequently no other
        clients can invoke this operation.
        
        The corresponding rpc call is 'idsm6_end_export_item_id_list_1_svc().'
        
        
        Use cases:
        At the moment this API is not used, and it may be added considering future requirement.
        """
        return cls.execute_soa_method(
            method_name='idsm6EndExportItemIdListSvc',
            library='Internal-MultiSite',
            service_date='2007_06',
            service_name='RemoteOperation',
            params={'inputs': inputs},
            response_cls=ServiceData,
        )

    @classmethod
    def idsm6EndSendObjectsSvc(cls, inputs: IDSM6EndSendObjectsIn) -> ServiceData:
        """
        The 'idsm6EndSendObjectsSvc' ends the system object s remote import process. The operation gets called, if
        abort flag is set to true by the previous operation. 
        
        This operation is integrated with Multi-Site Remote Procedure Call (RPC) framework and is callable only in
        Hypertext Transfer Protocol (HTTP) based Multi-Site environment. This operation acts as a wrapper call to
        server side RPC function and hence is invoked from within the Multi-Site framework. Consequently no other
        clients can invoke this operation.
        
        The corresponding RPC call is' idsm6_end_ send_objects_1_svc().'
        
        
        Use cases:
        This operation is called from the 'distributed_execute' utility. The utility is used to generate the item
        reports locally as well as remotely. To generate the report, internally this utility calls the 'item_report'
        utility.
        """
        return cls.execute_soa_method(
            method_name='idsm6EndSendObjectsSvc',
            library='Internal-MultiSite',
            service_date='2007_06',
            service_name='RemoteOperation',
            params={'inputs': inputs},
            response_cls=ServiceData,
        )

    @classmethod
    def idsm6PerformSendObjectsSvc(cls, inputs: IDSM6PerformSendObjectsIn) -> IDSM6PerformSendObjectsRes:
        """
        The 'idsm6PerformSendObjectsSvc' operation sends system objects to the remote site based on
        IDSM6PerformSendObjectsIn input.  This is the third operation in the sequence used in sending system objects
        such as Users, Groups, Persons, etc to the remote site. 
        
        This operation is integrated with Multi-Site Remote Procedure Call (RPC) framework and is callable only in HTTP
        based Multi-Site environment. This operation acts as a wrapper call to server side RPC function and hence is
        invoked from within the Multi-Site framework. Consequently no other clients can invoke this operation.
        
        Corresponding RPC call is 'idsm6_perform_send_objects_1_svc().'
        
        
        Use cases:
        'idsm6StartSendObjectsSvc' operation gets invoked from   dsa_util  utility while sending system objects such as
        Users, Groups, Persons, etc to remote site.
        Operation sequence is as below
        - idsm6StartSendObjectsSvc
        - file transfer via DIST call
        - idsm6PerformSendObjectsSvc  
        - idsm6EndSendObjectsSvc
        
        """
        return cls.execute_soa_method(
            method_name='idsm6PerformSendObjectsSvc',
            library='Internal-MultiSite',
            service_date='2007_06',
            service_name='RemoteOperation',
            params={'inputs': inputs},
            response_cls=IDSM6PerformSendObjectsRes,
        )

    @classmethod
    def idsm6StartExportItemIdListSvc(cls, inputs: IDSM6StartExportItemIdListIn) -> IDSM6StartExportItemIdListRes:
        """
        The 'idsm6StartExportItemIdListSvc' operation gets information about the server side staging directory and
        output export files. 
        
        This is the first operation in the sequence and cannot be called in isolation. 
        
        This operation is integrated with Multi-Site Remote Procedure Call (RPC) framework and is callable only in HTTP
        based Multi-Site environment. This operation acts as a wrapper call to server side RPC function and hence is
        invoked from within the Multi-Site framework. Consequently no other clients can invoke this operation.
                      
        Corresponding RPC call is 'idsm6_start_export_item_id_list_1_svc().'
        
        
        Use cases:
        'idsm6StartExportItemIdListSvc' operation is called from data_share utility while finding duplicate itemids at
        the remote site.
        """
        return cls.execute_soa_method(
            method_name='idsm6StartExportItemIdListSvc',
            library='Internal-MultiSite',
            service_date='2007_06',
            service_name='RemoteOperation',
            params={'inputs': inputs},
            response_cls=IDSM6StartExportItemIdListRes,
        )

    @classmethod
    def idsm6StartSendObjectsSvc(cls, inputs: IDSM6StartSendObjectsIn) -> IDSM6StartSendObjectsRes:
        """
        The 'idsm6StartSendObjectsSvc' operation gets information about the server side staging directory i.e. where
        the files are being transferred on the server side while sending system objects to the remote site. 
        
        This is the first operation in the sequence which gets information about the Multi-Site staging directory and
        cannot be called in isolation. 
        
        This operation is integrated with Multi-Site Remote Procedure Call (RPC) framework and is callable only in HTTP
        based Multi-Site environment. This operation acts as a wrapper call to server side RPC function and hence is
        invoked from within the Multi-Site framework. Consequently no other clients can invoke this operation.
        
        Corresponding RPC call is' idsm6_start_send_objects_1_svc().'
        
        
        Use cases:
        'idsm6StartSendObjectsSvc' operation gets invoked from   dsa_util  utility while sending system objects such as
        Users, Groups, Persons, etc to the remote site.
                Operation sequence is as below
        - idsm6StartSendObjectsSvc
        - file transfer via Distributed (DIST) ITK call
        - idsm6PerformSendObjectsSvc  
        - idsm6EndSendObjectsSvc
        
        """
        return cls.execute_soa_method(
            method_name='idsm6StartSendObjectsSvc',
            library='Internal-MultiSite',
            service_date='2007_06',
            service_name='RemoteOperation',
            params={'inputs': inputs},
            response_cls=IDSM6StartSendObjectsRes,
        )

    @classmethod
    def idsm7AskVersionSvc(cls, inputs: IDSM7AskVersionIn) -> IDSM7AskVersionRes:
        """
        The 'idsm7AskVersionSvc' operation gets the idsm server version information.
        
        This operation is integrated with Multi-Site Remote Procedure Call (RPC) framework and is callable only in HTTP
        based Multi-Site environment. This operation acts as a wrapper call to server side RPC function and hence is
        invoked from within the Multi-Site framework. Consequently no other clients can invoke this operation.
        
        Corresponding RPC call is 'idsm7_ask_version_1_svc().'
        
        
        Use cases:
        'idsm7AskVersionSvc' operation is called during remote check-in operation.
        """
        return cls.execute_soa_method(
            method_name='idsm7AskVersionSvc',
            library='Internal-MultiSite',
            service_date='2007_06',
            service_name='RemoteOperation',
            params={'inputs': inputs},
            response_cls=IDSM7AskVersionRes,
        )

    @classmethod
    def idsm7DistributedAppSvc(cls, inputs: IDSM7DistributedAppIn) -> IDSM7DistributedAppRes:
        """
        This distributed call checks, if the Integrated Distributed Services Manager (IDSM) provides the supplied
        service to the client. The op codes are used to specify the operation. Please refer the 'IDSM7DistributedAppIn'
        structure for the list of op codes.
        
        This operation is integrated with Multi-Site Remote Procedure Call (RPC) framework and is callable only in
        Hypertext Transfer Protocol (HTTP) based Multi-Site environment. This operation acts as a wrapper call to
        server side RPC function and hence is invoked from within the Multi-Site framework. Consequently no other
        clients can invoke this operation.
        
        The corresponding RPC call is 'idsm7_distributed_app_1_svc().'
        
        
        Use cases:
        This operation is called from the' DIST_IDSM_commit_synchronous_remote_import()', to check if the commit
        synchronous remote import operation supported by IDSM server.
        """
        return cls.execute_soa_method(
            method_name='idsm7DistributedAppSvc',
            library='Internal-MultiSite',
            service_date='2007_06',
            service_name='RemoteOperation',
            params={'inputs': inputs},
            response_cls=IDSM7DistributedAppRes,
        )

    @classmethod
    def idsm7StartExportSvc(cls, inputs: IDSM7StartExportIn) -> IDSM7StartExportRes:
        """
        The 'idsm7StartExportSvc' operation exports business objects at the remote site during a Remote Import
        transaction. When a user at Source Site selects business objects (e.g, Item) for remote import, the request is
        sent to the Target Site to export these objects into object.meta file at the Target Site and transferred over
        to the Source Site (requesting site). IDSM service at the Target Site exports the objects and creates
        object.meta file. The 'idsm7StartExportSvc' is the first operation in the sequence and hence cannot be called
        in isolation.
        
        This operation is integrated with Multi-Site Remote Procedure Call (RPC) framework and is callable only in HTTP
        based Multi-Site environment. This operation acts as a wrapper call to server side RPC function and hence is
        invoked from within the Multi-Site framework. Consequently no other clients can invoke this operation.
        
        Corresponding RPC call is' idsm7_start_export_1_svc().'
        
        
        Use cases:
        To remote import the business object, first the source site user creates an Item and publishes it to the
        default Object Directory Service (ODS) site.  The target site user performs remote search with the item id and
        imports it to the target site using the remote import operation. On successful completion of remote import
        operation, the item is imported to the target site database. To Remote Import the business object, the series
        of operations needs to be performed and this operation gets invoked as a first step in the Remote Import
        transaction. 
        The sequence of operations is as below 
        - idsm7StartExportSvc
        - idsm3ExportStatusSvc
        - idsm4XferExportFileSvc
        - idsm1EndExportSvc
        
        """
        return cls.execute_soa_method(
            method_name='idsm7StartExportSvc',
            library='Internal-MultiSite',
            service_date='2007_06',
            service_name='RemoteOperation',
            params={'inputs': inputs},
            response_cls=IDSM7StartExportRes,
        )

    @classmethod
    def idsm9DescribeObjectSvc(cls, inputs: IDSM9DescribeObjectIn) -> IDSM9DescribeObjectRes:
        """
        The 'idsm9DescribeObjectSvc' operation used to retrieve the object description from IDSM server.  Consumer
        should provide the client information and the tag of the object, as an input.
        
        This operation is integrated with Multi-Site Remote Procedure Call (RPC) framework and is callable only in
        Hypertext Transfer Protocol (HTTP) based Multi-Site environment. This operation acts as a wrapper call to
        server side RPC function and hence is invoked from within the Multi-Site framework. Consequently no other
        clients can invoke this operation.
        
        The corresponding RPC call is 'idsm9_describe_object_1_svc().'
        
        
        Use cases:
        To import the remote objects at the local site, object(s) published to the Object Directory Service (ODS) needs
        to be first searched and then imported. While loading the ODS published remote objects, this operation is
        called to get the object information.
        """
        return cls.execute_soa_method(
            method_name='idsm9DescribeObjectSvc',
            library='Internal-MultiSite',
            service_date='2007_06',
            service_name='RemoteOperation',
            params={'inputs': inputs},
            response_cls=IDSM9DescribeObjectRes,
        )


class ObjectDirectoryService(TcService):

    @classmethod
    def locatePublishedObjects(cls, clientInfo: ClientInfoProperties, objTagAsString: List[str]) -> LocatePublishedObjectResponse:
        """
        The 'locatePublishedObjects' operation locates published objects from the Object Directory Service (ODS) site
        for a given list of Tags.
        
        This operation is integrated with Multi-Site Remote Procedure Call (RPC) framework and is callable only in an
        HTTP based Multi-Site environment. This operation acts as a wrapper call to server side RPC function and hence
        is invoked from within the Multi-Site framework. Consequently no other clients can invoke this operation.
        
        Corresponding RPC call is 'ods9_locate_object_1_svc()'.
        
        Use cases:
        'locatePublishedObjects' operation gets called while importing stub objects from PSE (Product Structure
        Editor). It tries to locate the stub object by finding its owning site by checking publication record for that
        object. Export Assembly from site1 to site2 with including BOM. On site2, open assembly in PSE and it shows
        components as Remote Object(s), do import on that object(s), which tries to find the owning site of the object.
        """
        return cls.execute_soa_method(
            method_name='locatePublishedObjects',
            library='Internal-MultiSite',
            service_date='2007_06',
            service_name='ObjectDirectory',
            params={'clientInfo': clientInfo, 'objTagAsString': objTagAsString},
            response_cls=LocatePublishedObjectResponse,
        )

    @classmethod
    def publishObjects(cls, clientInfo: ClientInfoProperties, records: List[PublicationRecordProperties]) -> ServiceData:
        """
        The 'publishObjects' operation creates the 'PublicationRecords' at the Object Directory Service (ODS) site for
        each 'PublicationRecordProperties' input given.  
         
        This operation is integrated with Multi-Site Remote Procedure Call (RPC) framework and is callable only in an
        HTTP based Multi-Site environment. This operation acts as a wrapper call to server side RPC function and hence
        is invoked from within the Multi-Site framework. Consequently no other clients can invoke this operation.
        
        Corresponding RPC call is 'ods9_publish_object_1_svc().'
        
        
        Use cases:
        'publishObjects' operation gets called  in ODS publish object operation. The business object(s) can be shared
        within the Multi-Site federation. To remote import (Pull case) it must be published to the ODS. It can either
        be called from Rich Client (RAC) or from command line data_share (publish switch) utility.
        
        """
        return cls.execute_soa_method(
            method_name='publishObjects',
            library='Internal-MultiSite',
            service_date='2007_06',
            service_name='ObjectDirectory',
            params={'clientInfo': clientInfo, 'records': records},
            response_cls=ServiceData,
        )

    @classmethod
    def queryItemIdRecords(cls, clientInfo: ClientInfoProperties, itemId: List[str]) -> QueryItemIdRecordResponse:
        """
        The 'queryItemIdRecords' operation queries the ItemId Records from the central itemid registry for each
        'BoMfkInfo' input given. It checks whether input itemids are registered or not.
        
        This operation is integrated with Multi-Site Remote Procedure Call (RPC) framework and is callable only in an
        HTTP basedMulti-Site environment. This operation acts as a wrapper call to server side RPC function and hence
        is invoked from within the Multi-Site framework. Consequently no other clients can invoke this operation.
        
        Corresponding RPC call is 'ods9_describe_item_1_svc().'
        
        Use cases:
        'queryItemIdRecords' operation gets invoked in below use cases check whether given itemid is registered or not.
        - New Item creation when auto register is ON.
        - Registering  Item to  registry.
        
        """
        return cls.execute_soa_method(
            method_name='queryItemIdRecords',
            library='Internal-MultiSite',
            service_date='2007_06',
            service_name='ObjectDirectory',
            params={'clientInfo': clientInfo, 'itemId': itemId},
            response_cls=QueryItemIdRecordResponse,
        )

    @classmethod
    def queryPublicationRecords(cls, clientInfo: ClientInfoProperties, queryString: str, sortKeys: List[str], sortOrder: List[int]) -> QueryPublicationRecordsResponse:
        """
        The 'queryPublicationRecords' operation queries the PublicationRecords from the Object Directory Service (ODS)
        site based on the input query String. This query string should be in POM SQL query format.
        
        This operation is integrated with Multi-Site Remote Procedure Call (RPC) framework and is callable only in an
        HTTP based Multi-Site environment. This operation acts as a wrapper call to server side RPC function and hence
        is invoked from within the Multi-Site framework. Consequently no other clients can invoke this operation.
        
        Corresponding RPC call is 'ods9_execute_query_1_svc().'
        
        
        Use cases:
        'queryPublicationRecords' operation is called in ODS Remote search operation from the Rich Client (RAC) .It
        searches the ODS server for published objects that satisfy the search criteria.
        """
        return cls.execute_soa_method(
            method_name='queryPublicationRecords',
            library='Internal-MultiSite',
            service_date='2007_06',
            service_name='ObjectDirectory',
            params={'clientInfo': clientInfo, 'queryString': queryString, 'sortKeys': sortKeys, 'sortOrder': sortOrder},
            response_cls=QueryPublicationRecordsResponse,
        )

    @classmethod
    def unpublishObjects(cls, clientInfo: ClientInfoProperties, objTagAsString: List[str]) -> ServiceData:
        """
        The 'unpublishObjects' operation deletes the PublicationRecord from the Object Directory Service (ODS) site for
        each 'objTagAsString' input given.  
        
        This operation is integrated with Multi-Site Remote Procedure Call (RPC) framework and is callable only in an
        HTTP based Multi-Site environment. This operation acts as a wrapper call to server side RPC function and hence
        is invoked from within the Multi-Site framework. Consequently no other clients can invoke this operation.
        
        Corresponding RPC call is 'ods9_unpublish_object_1_svc().'
        
        Use cases:
        'unpublishObjects' operation gets called in ODS unpublish object operation. In Multi-Site federation to remote
        import (Pull case) the Business Object (BO), it must be published to the ODS. And to revoke the sharing, the
        business object should be unpublished from ODS. It can either be called from Rich Client (RAC) or from command
        line data_share (unpublish switch) utility.
        
        """
        return cls.execute_soa_method(
            method_name='unpublishObjects',
            library='Internal-MultiSite',
            service_date='2007_06',
            service_name='ObjectDirectory',
            params={'clientInfo': clientInfo, 'objTagAsString': objTagAsString},
            response_cls=ServiceData,
        )

    @classmethod
    def updatePublicationRecords(cls, clientInfo: ClientInfoProperties, records: List[PublicationRecordProperties]) -> ServiceData:
        """
        The 'updatePublicationRecords' opertaion updates the PublicationRecord at the Object Directory Service (ODS)
        site for each 'PublicationRecordProperties' input given.  
        
        This operation is integrated with Multi-Site Remote Procedure Call (RPC) framework and is callable only in HTTP
        based Multi-Site environment. This operation acts as a wrapper call to server side RPC function and hence is
        invoked from within the Multi-Site framework. Consequently no other clients can invoke this operation.
        
        Corresponding RPC call is 'ods9_refresh_object_1_svc().'
        
        
        Use cases:
        'updatePublicationRecords' operation gets invoked in Integrated Distributed Service Manager (IDSM) re-publish
        object operation after object is exported to other site with ownership transfer in use cases like remote
        import/export. The ownership transfer operation performed on the business objects (BOs) change its ownership.
        After transfer of ownership, if it is not re-published then the BO has the wrong entry for the owning siteId in
        PublicationRecord at ODS site. In order to correct it, the BOs needs to be republished to the ODS server.
        """
        return cls.execute_soa_method(
            method_name='updatePublicationRecords',
            library='Internal-MultiSite',
            service_date='2007_06',
            service_name='ObjectDirectory',
            params={'clientInfo': clientInfo, 'records': records},
            response_cls=ServiceData,
        )

    @classmethod
    def createItemIdRecords(cls, clientInfo: ClientInfoProperties, records: List[ItemIdRecordsProperties]) -> ServiceData:
        """
        The 'createItemIdRecords' operation creates Item Records in the item registry based on
        'ItemIdRecordsProperties' input.
        
        This operation is integrated with Multi-Site Remote Procedure Call ( RPC) framework and is callable only in an
        HTTP based Multi-Site environment. This operation acts as a wrapper call to server side RPC function and hence
        is invoked from within the Multi-Site framework. Consequently no other clients can invoke this operation.
        
        Corresponding RPC call is 'ods9_register_item_1_svc()'.
        
        
        Use cases:
        - Admin user(s) set the 'ITEM_id_always_register_on_creation' to TRUE on all sites within a Multi-Site
        federation. On item creation the items created automatically registers with the central item registry.
        - Admin user(s) can register existing item(s) with the central item registry, using Register 
        
        """
        return cls.execute_soa_method(
            method_name='createItemIdRecords',
            library='Internal-MultiSite',
            service_date='2007_06',
            service_name='ObjectDirectory',
            params={'clientInfo': clientInfo, 'records': records},
            response_cls=ServiceData,
        )

    @classmethod
    def deleteItemIdRecords(cls, clientInfo: ClientInfoProperties, itemId: List[str]) -> ServiceData:
        """
        The 'deleteItemIdRecords' operation deletes Item Records from the item registry for each 'BoMfkInfo' input.
        
        This operation is integrated with Multi-Site Remote Procedure Call ( RPC) framework and is callable only in an
        HTTP based Multi-Site environment. This operation acts as a wrapper call to server side RPC function and hence
        is invoked from within the Multi-Site framework. Consequently no other clients can invoke this operation.
        
        Corresponding RPC call is 'ods9_unregister_item_1_svc()'.
        
        
        Use cases:
        Admin user(s) can unregister existing item(s) from the central item registry, using Unregister Item Id command
        in the Rich Client (RAC) or command line utility data_share  f=unregister.
        """
        return cls.execute_soa_method(
            method_name='deleteItemIdRecords',
            library='Internal-MultiSite',
            service_date='2007_06',
            service_name='ObjectDirectory',
            params={'clientInfo': clientInfo, 'itemId': itemId},
            response_cls=ServiceData,
        )

    @classmethod
    def describePublicationRecords(cls, clientInfo: ClientInfoProperties, publishedObjTagsAsString: List[str]) -> DescribePublicationRecordsResponse:
        """
         The 'describePublicationRecords' operation returns the 'PublicationRecord' properties for a given list of
        published objects. 
         
        This operation is integrated with Multi-Site Remote Procedure Call ( RPC) framework and is callable only in an
        HTTP based Multi-Site environment. This operation acts as a wrapper call to server side RPC function and hence
        is invoked from within the Multi-Site framework. Consequently no other clients can invoke this operation.
        
        Corresponding RPC call is 'ods9_describe_object_1_svc().'
        
        
        Use cases:
        'describePublicationRecords' operation gets called via Object Directory Service (ODS) remote search in HTTP
        Multi-Site environment.  ODS Remote search returns list of the Published objects as per search criteria. For
        the list of published objects, describePublicationRecords operation is called to get the publicationRecord
        properties on each object.
        """
        return cls.execute_soa_method(
            method_name='describePublicationRecords',
            library='Internal-MultiSite',
            service_date='2007_06',
            service_name='ObjectDirectory',
            params={'clientInfo': clientInfo, 'publishedObjTagsAsString': publishedObjTagsAsString},
            response_cls=DescribePublicationRecordsResponse,
        )

    @classmethod
    def distributedApplication(cls, clientInfo: ClientInfoProperties, distAppProps: List[DistributedAppProperties]) -> DistributedAppResponse:
        """
        The 'distributedApplication' operation is a generic operation which is corresponding to its counterpart in
        Remote Procedure Call (RPC).
        
        This operation is integrated with Multi-Site RPC framework and is callable only in HTTP based Multi-Site
        environment. This operation acts as a wrapper call to server side RPC function and hence is invoked from within
        the Multi-Site framework. Consequently no other clients can invoke this operation.
        
        Corresponding RPC call is 'ods9_distributed_app_1_svc().'
        
        
        Use cases:
        'distributedApplication' operation id used by Distributed (DIST) ITKs to send an RPC call.
        """
        return cls.execute_soa_method(
            method_name='distributedApplication',
            library='Internal-MultiSite',
            service_date='2007_06',
            service_name='ObjectDirectory',
            params={'clientInfo': clientInfo, 'distAppProps': distAppProps},
            response_cls=DistributedAppResponse,
        )

    @classmethod
    def getODSVersion(cls, clientInfo: ClientInfoProperties) -> GetODSVersionResponse:
        """
        The 'getODSVersion' operation gets the Object Directory Service (ODS) server version information.
        
        This operation is integrated with Multi-Site Remote Procedure Call (RPC) framework and is callable only in HTTP
        based Multi-Site environment. This operation acts as a wrapper call to server side RPC function and hence is
        invoked from within the Multi-Site framework. Consequently no other clients can invoke this operation.
        
        Corresponding RPC call is 'ods9_version_check_1_svc().'
        
        
        Use cases:
        'getODSVersion' operation gets invoked prior to send any ODS requests to ODS server for getting ODS version for
        Multi-site compatibility check.
        """
        return cls.execute_soa_method(
            method_name='getODSVersion',
            library='Internal-MultiSite',
            service_date='2007_06',
            service_name='ObjectDirectory',
            params={'clientInfo': clientInfo},
            response_cls=GetODSVersionResponse,
        )


class TcEntObjectDirectoryService(TcService):

    @classmethod
    def publishObjects(cls, clientInfo: TcEntClientInfoProperties, records: List[TcEntPublicationRecordProperties]) -> ServiceData:
        """
        The publishObjects operation creates the PublicationRecords on the Object Directory Service (ODS) site for each
        TcEntPublicationRecordProperties input supplied.
        
        This operation is integrated with Multi-Site Remote Procedure Call (RPC) framework and is callable only in
        Hypertext Transfer Protocol (HTTP) based Multi-Site environment. This operation acts as a wrapper call to
        server side RPC function and hence is invoked from within the Multi-Site framework. Consequently no other
        clients can invoke this operation.
        
        
        
        Use cases:
        The business objects can be shared within the Multi-Site federation. To share it must be published to the
        Object Directory Services (ODS) or exported to the remote site. The 'publishObjects' operation is called while
        publishing the business object to ODS; the Teamcenter Enterprise client calls it.
        """
        return cls.execute_soa_method(
            method_name='publishObjects',
            library='Internal-MultiSite',
            service_date='2007_06',
            service_name='TcEntObjectDirectory',
            params={'clientInfo': clientInfo, 'records': records},
            response_cls=ServiceData,
        )

    @classmethod
    def queryPublicationRecords(cls, clientInfo: TcEntClientInfoProperties, queryParams: List[QueryPublicationRecordsProperties]) -> QueryPublicationRecordsResponse:
        """
        The 'queryPublicationRecords' operation queries the PublicationRecords from the Object Directory Service (ODS)
        server based on the input query String. This query string should be in POM SQL query format.
        
        This operation is integrated with Multi-Site Remote Procedure Call (RPC) framework and is callable only in
        Hypertext Transfer Protocol (HTTP) based Multi-Site environment. This operation acts as a wrapper call to
        server side RPC function and hence is invoked from within the Multi-Site framework. Consequently no other
        clients can invoke this operation.
        
        Use cases:
        Within a Multi-Site federation the object published to ODS server can be remote searched from the Rich Client
        (RAC). The 'queryPublicationRecords' operation is called in remote search operation. At ODS server it searches
        for published objects that satisfy the search criteria.
        """
        return cls.execute_soa_method(
            method_name='queryPublicationRecords',
            library='Internal-MultiSite',
            service_date='2007_06',
            service_name='TcEntObjectDirectory',
            params={'clientInfo': clientInfo, 'queryParams': queryParams},
            response_cls=QueryPublicationRecordsResponse,
        )

    @classmethod
    def republishObjectAfterOwnershipTransfer(cls, clientInfo: TcEntClientInfoProperties, republishParams: List[RepublishObjectInputProperties]) -> ServiceData:
        """
        The 'republishObjectAfterOwnershipTransfer' operation is used by Teamcenter Enterprise client to re-publish the
        objects, after ownership transfer performed. It updates the PublicationRecord on the Object Directory Services
        (ODS) server for each 'RepublishObjectInputProperties' input set of objects.
        
        This operation is integrated with Multi-Site Remote Procedure Call (RPC) framework and is callable only in
        Hypertext Transfer Protocol (HTTP) based Multi-Site environment. This operation acts as a wrapper call to
        server side RPC function and hence is invoked from within the Multi-Site framework. Consequently no other
        clients can invoke this operation.
        
        
        Use cases:
        The ownership transfer operation performed on the business objects (BOs) change its ownership. After transfer
        of ownership, if it is not re-published then the BO has the wrong entry for the owner. In order to correct it,
        the BOs needs to be republished to the ODS server. 'republishObjectAfterOwnershipTransfer' operation is called
        while BOs republished to the ODS server.
        """
        return cls.execute_soa_method(
            method_name='republishObjectAfterOwnershipTransfer',
            library='Internal-MultiSite',
            service_date='2007_06',
            service_name='TcEntObjectDirectory',
            params={'clientInfo': clientInfo, 'republishParams': republishParams},
            response_cls=ServiceData,
        )

    @classmethod
    def unpublishObjects(cls, clientInfo: TcEntClientInfoProperties, objTagsAsString: List[str]) -> ServiceData:
        """
        The 'unpublishObjects' operation deletes the PublicationRecord from the Object Directory Service  (ODS) site
        for each 'objTagAsString' input set of objects.
        
        This operation is integrated with Multi-Site Remote Procedure Call (RPC) framework and is callable only in
        Hypertext Transfer Protocol (HTTP) based Multi-Site environment. This operation acts as a wrapper call to
        server side RPC function and hence is invoked from within the Multi-Site framework. Consequently no other
        clients can invoke this operation.
        
        
        Use cases:
        The business objects can be shared within the Multi-Site federation. To share, it must be published to the
        Object Directory Services (ODS) or exported to the remote site. And to revoke the sharing, the business object
        should be unpublished from ODS. The 'unpublishObjects' operation is called while unpublishing the business
        object from ODS; the Teamcenter Enterprise client calls it.
        """
        return cls.execute_soa_method(
            method_name='unpublishObjects',
            library='Internal-MultiSite',
            service_date='2007_06',
            service_name='TcEntObjectDirectory',
            params={'clientInfo': clientInfo, 'objTagsAsString': objTagsAsString},
            response_cls=ServiceData,
        )

    @classmethod
    def describePublicationRecords(cls, clientInfo: TcEntClientInfoProperties, publishedObjTagAsString: List[str]) -> DescribePublicationRecordsResponse:
        """
        The 'describePublicationRecords' operation returns the PublicationRecord of published objects based on the
        TcEnterprise client information and the published object tag.
        
        This operation is integrated with Multi-Site Remote Procedure Call (RPC) framework and is callable only in
        Hypertext Transfer Protocol (HTTP) based Multi-Site environment. This operation acts as a wrapper call to
        server side RPC function and hence is invoked from within the Multi-Site framework. Consequently no other
        clients can invoke this operation.
        
        Use cases:
        The 'describePublicationRecords' operation gets called via Object Directory Service (ODS) remote search in HTTP
        Multi-Site environment.  ODS Remote search returns list of the published objects as per search criteria. For
        the list of published object, describePublicationRecords operation is called to get the PublicationRecord
        properties on each object.
        """
        return cls.execute_soa_method(
            method_name='describePublicationRecords',
            library='Internal-MultiSite',
            service_date='2007_06',
            service_name='TcEntObjectDirectory',
            params={'clientInfo': clientInfo, 'publishedObjTagAsString': publishedObjTagAsString},
            response_cls=DescribePublicationRecordsResponse,
        )

    @classmethod
    def getODSVersion(cls, clientInfo: TcEntClientInfoProperties) -> GetODSVersionResponse:
        """
        The 'getODSVersion' returns the Object Directory Services (ODS) server release information, for the requesting
        TcEnterprise client information.
        
        This operation is integrated with Multi-Site Remote Procedure Call (RPC) framework and is callable only in
        Hypertext Transfer Protocol (HTTP) based Multi-Site environment. This operation acts as a wrapper call to
        server side RPC function and hence is invoked from within the Multi-Site framework. Consequently no other
        clients can invoke this operation.
        
        Use cases:
        Multi-site federation supports the data transfer between different Teamcenter versions, as well with the
        Teamcenter Enterprise. To check the compatibility the 'getODSVersion' operation invoked for checking the
        version of ODS server.
        """
        return cls.execute_soa_method(
            method_name='getODSVersion',
            library='Internal-MultiSite',
            service_date='2007_06',
            service_name='TcEntObjectDirectory',
            params={'clientInfo': clientInfo},
            response_cls=GetODSVersionResponse,
        )



from __future__ import annotations

from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from tcsoa.gen.BusinessObjects import WorkspaceObject
from dataclasses import dataclass


@dataclass
class FindAffectedCCsOutput(TcBaseObj):
    """
    The output object for FindAffectedCCs SOA.
    This object contains a list of FindAffectedOutputObject. This vector represents all of the queried objects and all
    of the affected CCs.
    
    :var affectedObjects: Each vector node inside "affectedObjects" includes  a quaried item and a single Collaboration
    Context. For example, if CC1 (Collaboration Context) and CC2 (Collaboration Context) contain inside their process
    structures both item1 and item 2 and the two items are quaried for affected Collaboration Contexts, the output will
    be as follows:
    1. Item1, CC1
    2. Item1, CC2
    3. Item2, CC1
    4. Item2, CC2
    
    Each of the items is represented by two vector nodes, one vector node for each relation to a CC.
    """
    affectedObjects: List[FindAffectedOutputObject] = ()


@dataclass
class FindAffectedCCsResponse(TcBaseObj):
    """
    A list of output objects and a service data.
    
    :var output: The output data includes the affected CCs and the items which is related to each CC.
    :var serviceData: The service data output.
    """
    output: List[FindAffectedCCsOutput] = ()
    serviceData: ServiceData = None


@dataclass
class FindAffectedOutputObject(TcBaseObj):
    """
    The output object which contains the queried object and one of it's related CCs
    
    :var queryObject: The item queried for affected CCs.
    :var affectedCC: One of the affected CCs of the query item.
    """
    queryObject: WorkspaceObject = None
    affectedCC: WorkspaceObject = None



from __future__ import annotations

from tcsoa.gen.Query._2018_11.SavedQuery import BusinessObjectQueryInput2
from typing import List
from tcsoa.gen.Query._2007_09.SavedQuery import SavedQueriesResponse
from tcsoa.utils import TcService


class SavedQueryService(TcService):

    @classmethod
    def executeBOQueriesWithContext(cls, inputs: List[BusinessObjectQueryInput2]) -> SavedQueriesResponse:
        """
        Execute business object searches (Simple Search) and return search results. 
        
        This operation is invoked to query business object instances including Logical Object instances. The input
        structure identifies the type, clauses, and max number to return for the search. Additionally, a map of of
        property name and ConfigurationContext pairs are input to support configuration context as a filter for the
        search.
        
        Use cases:
        This operation is invoked to query business object instances including Logical Object instances.
        
        The Logical Object query use cases are described below: 
        
        Logical Object Query Use Case Data:
        
        1.    Logical object definition 1
            Name                                            :     "LogicalItem"
            Root business object                :      Item
            memberID1                                :     ItemRevision
            Included logical object ID1        :    "LogicalItemRevisionDomainA"
        
        2.    Logical object definition 2
            Name                                            :     "LogicalItemRevisionDomainA"
            Root business object                    :      ItemRevision
        
        
        
         Use Case 1: 
         
        BusinessObjectQueryInput2 [0] = { 
            typeName                       =      "LogicalItem", 
            clauses [0]= 
                { 
                    propName = "object_name",
                    propValue = "design1",
                    mathOperator = "=",
                    logicOperator = "AND"
                  }
            maxNumToReturn         =     30,
            requestId             =     1,
            clientId             =     1,
        }
        
        This operation will return instances of "LogicalItem", object_name of root "Item" is "design1". Since no
        configuration context is provided for "Included logical object ID1", all the instances of 
        "LogicalItemRevisionDomainA" will be returned.
        
        The presented properties on the instances of "LogicalItem" and its included logical object:
        "LogicalItemRevisionDomainA" will be returned through Service Data. 
        
        Use Case 2:
        
        BusinessObjectQueryInput2 [0] = { 
            typeName                       =      "LogicalItem", 
            clauses [0]= 
            { 
                    propName = "object_name",
                propValue = "design1",
                mathOperator = "=",
                logicOperator = "AND"
              }
            maxNumToReturn         =     30,
            requestId             =     1,
            clientId             =     1,
            configurationContextMap =           [<"Included logical object ID1", "ConfigContextUid1">]
        }
        
        This operation will return instances of "Logical Object 1", object_name of root "Item" is "design1". Since the
        configuration context is provided for "Included logical object ID1", one instance of 
        "LogicalItemRevisionDomainA" will be returned for the root Item Revision which satisfies the input
        configuration context criteria.
        
        The presented properties on the instances of "LogicalItem" and its included logical object:
        "LogicalItemRevisionDomainA" will be returned through Service Data.
        
        Use Case 3:
        
        BusinessObjectQueryInput2 [0] = { 
            typeName                       =      "LogicalItem", 
            clauses [0]= 
            { 
                    propName = "object_name",
                propValue = "design1",
                mathOperator = "=",
                logicOperator = "AND"
              }
            maxNumToReturn         =     30,
            requestId             =     1,
            clientId             =     1,
            configurationContextMap =           [<"memberID1", "ConfigContextUid2">]
        }
        
        This operation will return instances of "Logical Object 1", object_name of root "Item" is "design1". Since the
        configuration context is provided for "memberID1", one instance of "ItemRevision" will be returned for the root
        Item which satisfies the input configuration context criteria.
        
        The presented properties on the instances of "LogicalItem" and its included logical object:
        "LogicalItemDomainA" will be returned through Service Data.
        """
        return cls.execute_soa_method(
            method_name='executeBOQueriesWithContext',
            library='Query',
            service_date='2018_11',
            service_name='SavedQuery',
            params={'inputs': inputs},
            response_cls=SavedQueriesResponse,
        )



from __future__ import annotations

from tcsoa.gen.Internal.ActiveWorkspaceBom._2018_05.OccurrenceManagement import RemoveSubstitutesData, DuplicateAndReplaceData, AddSubstitutesData
from tcsoa.gen.BusinessObjects import BusinessObject, Awb0Element, WorkspaceObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class OccurrenceManagementService(TcService):

    @classmethod
    def addSubstitutes(cls, inputData: AddSubstitutesData) -> ServiceData:
        """
        Adds objects as substitutes to the selected object. Substitute components are parts that are interchangeable
        with a particular component in an assembly.
        """
        return cls.execute_soa_method(
            method_name='addSubstitutes',
            library='Internal-ActiveWorkspaceBom',
            service_date='2018_05',
            service_name='OccurrenceManagement',
            params={'inputData': inputData},
            response_cls=ServiceData,
        )

    @classmethod
    def duplicateAndReplace(cls, inputData: DuplicateAndReplaceData) -> ServiceData:
        """
        This operation creates duplicate of input Awb0Element objects and structure below it. Input Awb0Element objects
        gets replaced with newly created duplicate objects.
        """
        return cls.execute_soa_method(
            method_name='duplicateAndReplace',
            library='Internal-ActiveWorkspaceBom',
            service_date='2018_05',
            service_name='OccurrenceManagement',
            params={'inputData': inputData},
            response_cls=ServiceData,
        )

    @classmethod
    def duplicateAndReplaceAsync(cls, inputData: DuplicateAndReplaceData) -> None:
        """
        This operation creates duplicate of input Awb0Element objects and structure below it. Input Awb0Element objects
        get replaced with newly created duplicate objects asynchronously. The user can perform other tasks while the
        operation will run in background. After the task is performed, the method will create a message and send it as
        a notification to the user performing the task.
        """
        return cls.execute_soa_method(
            method_name='duplicateAndReplaceAsync',
            library='Internal-ActiveWorkspaceBom',
            service_date='2018_05',
            service_name='OccurrenceManagement',
            params={'inputData': inputData},
            response_cls=None,
        )

    @classmethod
    def preferSubstitute(cls, occurrence: BusinessObject, preferredSubstitute: WorkspaceObject) -> ServiceData:
        """
        Sets the object as preferred for the input occurrence. The content is saved during the operation.
        
        Use cases:
        In an assembly there can be multiple substitutes of an occurrence. From the substitutes there can be one
        preferred substitue, the user can call preferSubstitute operation and pass in the occurrence and selected
        substitute to mark as preferred substitute.
        """
        return cls.execute_soa_method(
            method_name='preferSubstitute',
            library='Internal-ActiveWorkspaceBom',
            service_date='2018_05',
            service_name='OccurrenceManagement',
            params={'occurrence': occurrence, 'preferredSubstitute': preferredSubstitute},
            response_cls=ServiceData,
        )

    @classmethod
    def removeElements(cls, elementsToRemove: List[Awb0Element]) -> ServiceData:
        """
        This operation removes Awb0Element objects from the content or Awb0SavedBookmark. 
        The product is implicitly saved after removing Awb0Element objects. When product(s) is removed from
        Awb0SavedBookmark user needs to explicitly save the changes.User must have WRITE privilege to remove elements.
        """
        return cls.execute_soa_method(
            method_name='removeElements',
            library='Internal-ActiveWorkspaceBom',
            service_date='2018_05',
            service_name='OccurrenceManagement',
            params={'elementsToRemove': elementsToRemove},
            response_cls=ServiceData,
        )

    @classmethod
    def removeSubstitutes(cls, inputData: RemoveSubstitutesData) -> ServiceData:
        """
        Removes specified substitutes for the input element. The changes are implicitly saved after removing
        substitutes. User must have WRITE privilege to remove substitutes.
        """
        return cls.execute_soa_method(
            method_name='removeSubstitutes',
            library='Internal-ActiveWorkspaceBom',
            service_date='2018_05',
            service_name='OccurrenceManagement',
            params={'inputData': inputData},
            response_cls=ServiceData,
        )



from __future__ import annotations

from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetVendorCatalogInfo3Response(TcBaseObj):
    """
    A list of catalog info objects that contain information about all valid vendor catalogs of the requested types that
    are available on the Teamcenter server machine.
    Each catalog info object from this list contains detailed information about the catalog (see below). It also
    contains a unique catalog ID that can be used to identify the catalog lateron. Its  root class ID identifies under
    which root class this vendor catalog will reside in Teamcenter's vendor catalog classification hierarchy once it is
    imported.
    The root directory information contains the absolute path of the directory where the catalog resides on the
    Teamcenter server. Only valid catalogs of the requested types will be returned. If a corrupted catalog is found,
    for example if it is missing mandatory files or directories, this catalog's information will not be returned in the
    list of catalog information objects. If this operation finds such corrupted catalogs, it continues searching for
    other valid catalogs and returns all valid catalogs of the requested types.
    
    :var serviceData: The service data.
    :var catalogInfo: A list of catalog infos.
    """
    serviceData: ServiceData = None
    catalogInfo: List[CatalogInfo3] = ()


@dataclass
class CatalogInfo3(TcBaseObj):
    """
    The structure contains vendor catalog information for GTC V1 and GTC V2.
    
    :var vendorName: The name of the tool vendor company providing the catalog.
    :var vendorAcronym: An acronym for the tool vendor providing the catalog, usually consisting of two or three
    characters.
    :var gtcHierarchyVersion: The Generic Tool Catalog version number. For GTC V1 packages the format is defined by
    "V<version>R<revision>", for example "V1R7"; for GTC V2 packages the format is for example "0.1.0".
    :var gtcPackageId: A unique identifier for this Generic Tool Catalog package.
    :var logoUrl: {GTC-V2 only} The filepath of the package logo.
    :var disclaimerText: {GTC-V2 only} The disclaimer text in the current Teamcenter language. If the disclaimer does
    not exist in the current language, the english text will be returned.
    :var gtcVersion: The version of this GTC package, for example "1" for V1 or "2" for V2.
    :var vendorHierarchyVersion: {GTC V2 only} The version of the GTC class hierarchy that is used in this package.
    :var downloadSecurity: {GTC V2 and online packages only} This value states if the tool vendor would like to set a
    control on package data release or not. Possible values are "yes" and "no". This is an optional value.
    :var onlineConnectionConfiguration: {GTC V2 and online packages only} This value is used for online data download
    only. It refers to a technical configuration file (an URL) that is provided by data senders who support online data
    download.
    :var vendorCatalogVersion: The version of the catalog.
    :var vendorCatalogLanguage: The language of the catalog.
    :var vendorCatalogDescription: A long description of the catalog. (For GTC V2 packages it is returned in the
    current Teamcenter server language.)
    :var vendorCatalogShortDescription: A short description of the catalog. (For GTC v2 packages it is returned in the
    current Teamcenter server language.)
    :var vendorCatalogId: A unique ID for the catalog.
    :var vendorCatalogRootClassId: The ID of the catalog's root class in the Generic Tool Catalog's Classification
    hierarchy when the catalog gets imported, typically the concatenation of the vendor's acronym and "#GTC" -> 
    "<vendorAcronym>#GTC".
    :var vendorCatalogRootDir: The root directory where the tool vendor catalog is stored on the Teamcenter server.
    :var gtcPackageCreationDate: The creation date of the Generic Tool Catalog package, in the format yyyymmdd_hhmmss.
    """
    vendorName: str = ''
    vendorAcronym: str = ''
    gtcHierarchyVersion: str = ''
    gtcPackageId: str = ''
    logoUrl: str = ''
    disclaimerText: str = ''
    gtcVersion: str = ''
    vendorHierarchyVersion: str = ''
    downloadSecurity: str = ''
    onlineConnectionConfiguration: str = ''
    vendorCatalogVersion: str = ''
    vendorCatalogLanguage: str = ''
    vendorCatalogDescription: str = ''
    vendorCatalogShortDescription: str = ''
    vendorCatalogId: str = ''
    vendorCatalogRootClassId: str = ''
    vendorCatalogRootDir: str = ''
    gtcPackageCreationDate: str = ''



from __future__ import annotations

from tcsoa.gen.Internal.Rdv._2007_09.VariantManagement import GetVariantExprAllDataResponse
from typing import List
from tcsoa.gen.BusinessObjects import NamedVariantExpression
from tcsoa.utils import TcService


class VariantManagementService(TcService):

    @classmethod
    def getVariantExprXOChartData(cls, inputs: List[NamedVariantExpression]) -> GetVariantExprAllDataResponse:
        """
        This operation creates the necessary data required to display the Named Variant Expressions (NVEs) in the XO
        mode. inputs parameter contains the list of NVE references for which XO chart data is required. XO mode is an
        alternate way to display the Named Variant Expression.  In XO mode X means true and O means false.
        
        Use cases:
        This operation is called when there is a need to fetch the XO chart of the NVEs. This operation takes the list
        of NVE objects and returns the 'GetVariantExprAllDataResponse' object which contains the data required to
        create the XO chart and 'ServiceData'.
        """
        return cls.execute_soa_method(
            method_name='getVariantExprXOChartData',
            library='Internal-Rdv',
            service_date='2007_09',
            service_name='VariantManagement',
            params={'inputs': inputs},
            response_cls=GetVariantExprAllDataResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class ItemIDData(TcBaseObj):
    """
    Structure containing an item_id and a reference to an existing object with that ID.
    
    :var clientId: A unique string supplied by the caller which comes through input data.
    :var itemID: Existing Item object with the same item_id. The presence of this object indicates that the item
    already exists in the system, or that the supplied combination of external 'objectID' and 'contextID' is not unique.
    :var existingObject: Object tag of an existing item with the same item_id.
    The presence of this tag indicates that the item
    already exists in the system, or that the supplied
    combination of external object ID and context ID is
    not unique.
    """
    clientId: str = ''
    itemID: str = ''
    existingObject: BusinessObject = None


@dataclass
class ItemIDInput(TcBaseObj):
    """
    A structure of external IDs and context IDs. This data is used to generate the Item IDs with context. It will also
    be used to compare and find out existing Item with such ID.
    
    :var clientId: A unique string supplied by the caller. This ID is used to identify returned 'ItemData' elements and
    Partial Errors associated with this 'ItemIDInput'.
    :var externalObjectID: The object ID provided from the external system. The assumption is that this ID is unique
    within the supplied context.
    :var contextID: An ID designating the external system, usually the ID of a vendor or a partner company providing
    the Item information.
    """
    clientId: str = ''
    externalObjectID: str = ''
    contextID: str = ''


@dataclass
class ItemIDResponse(TcBaseObj):
    """
    Response structure for 'getItemIDwithContext'
    
    :var itemData: A list of ItemIDData structures which essentially denote generated IDs with potential existing Item
    with the generated ID. Errors\Partial errors will sit with standard Teamcenter 'ServiceData' structure.
    :var serviceData: Standard 'ServiceData' structure. It contains partial errors and failures along with the
    clientIds.
    """
    itemData: List[ItemIDData] = ()
    serviceData: ServiceData = None


@dataclass
class LineItemPropsWithType(TcBaseObj):
    """
    The information about each BidPackageLineItem type to be processed is provided by way of the
    'LineItemPropsWithType' data structure. This structure accepts the BidPackageLineItem type to be processed.
    
    :var name: Name of the BidPackageLineItem to be created.
    :var description: Description of the BidPackageLineItem Object.
    :var quote: BidPackageQuote object to be added to BidPackageLineItem.
    :var bpliTypeName: BidPackageLineItem Type name to be used.
    :var liccname: Name of the BidPackageLineItemConfigContext to be created.
    :var liccdesc: Description of the BidPackageLineItemConfigContext to be created.
    :var partid: Id of the part to be associated with BidPackageLineItem.
    :var viewtype: PSView Type to be associated with BidPackageLineItemConfigContext.
    :var quantity: Quantity to be created for BidPackageLineItem objects.
    :var revRule: Revision rule to be associated with BidPackageLineItemConfigContext.
    :var varRule: Variant rule to be associated with BidPackageLineItemConfigContext.
    :var closureRule: Closure rule to be associated with BidPackageLineItemConfigContext.
    """
    name: str = ''
    description: str = ''
    quote: BusinessObject = None
    bpliTypeName: str = ''
    liccname: str = ''
    liccdesc: str = ''
    partid: str = ''
    viewtype: str = ''
    quantity: int = 0
    revRule: str = ''
    varRule: str = ''
    closureRule: str = ''



from __future__ import annotations

from tcsoa.gen.BusinessObjects import Schedule
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService
from tcsoa.gen.ProjectManagement._2017_11.ScheduleManagement import WhatIfAnalysisOption


class ScheduleManagementService(TcService):

    @classmethod
    def whatIfAnalysis(cls, schedule: Schedule, whatIfOption: WhatIfAnalysisOption) -> ServiceData:
        """
        This operation allows the user to perform What-If analysis on a Schedule by making scheduling changes locally
        without affecting other users and save or cancel the changes made during the What-If analysis.
        
        Only the following operations can be performed by the user who has started the What-If analysis:
        1. Modify all properties of the Schedule. 
        2. Modify all properties of the ScheduleTask other than the following execution proeprties : status, work
        complete, complete percent, actual start date, actual finish date, work remaining.
        3. Create, delete and update ScheduleTask, ResourceAssignment, TaskDependency, and Fnd0ProxyTask objects.
        
        The following operations cannot be performed by the user who has started the What-If analysis:
        1. Modify the following execution properties of the ScheduleTask : status, work complete, complete percent,
        actual start date, actual finish date, work remaining.
        2. Insert Schedule and detach Schedule operations.
        
        Only the following operations can be performed by other users when a Schedule is in What-If analysis mode:
        1. Modify the following execution properties of the ScheduleTask : status, work complete, complete percent,
        actual start date, actual finish date, work remaining.
        """
        return cls.execute_soa_method(
            method_name='whatIfAnalysis',
            library='ProjectManagement',
            service_date='2017_11',
            service_name='ScheduleManagement',
            params={'schedule': schedule, 'whatIfOption': whatIfOption},
            response_cls=ServiceData,
        )



from __future__ import annotations

from tcsoa.gen.Internal.Administration._2018_11.SiteManagement import CreateOrUpdateSitesResponse, SiteInfo
from typing import List
from tcsoa.utils import TcService


class SiteManagementService(TcService):

    @classmethod
    def createOrUpdateSites(cls, siteInfos: List[SiteInfo]) -> CreateOrUpdateSitesResponse:
        """
        This operation creates or updates POM_imc business objects with given properties. A new POM_imc business object
        will be created if it does not already exist, otherwise existing POM_imc business objects will be updated. In
        order to update, a valid Site ID must exist.
        
        The only required fields are Site Name and Site ID, additional properties can be added through the use of the
        additional properties map.
        
        All fields can be updated besides Site ID.
        
        Use cases:
        Use Case 1: Create a site based on the site ID, site name, and any additional properties.
        Use Case 2: Given a site ID that exists, update the site information with the site name and any additional
        properties
        """
        return cls.execute_soa_method(
            method_name='createOrUpdateSites',
            library='Internal-Administration',
            service_date='2018_11',
            service_name='SiteManagement',
            params={'siteInfos': siteInfos},
            response_cls=CreateOrUpdateSitesResponse,
        )



from tcsoa.gen.Internal_ConfigFilterCriteria._2013_12.services import VariantManagementService as imp0
from tcsoa.utils import TcService


class VariantManagementService(TcService):
    executeConfiguratorService = imp0.executeConfiguratorService



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from enum import Enum
from typing import Dict, List
from tcsoa.gen.Core._2008_06.PropDescriptor import PropDesc
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class PropDescSaveAs(TcBaseObj):
    """
    This structure captures a property definition for a 'SaveAs' Descriptor.
    
    :var parent: Core property descriptor structure
    :var copyFromOriginal: Indicating if the property value is to be copied from original object to the object copy - 
    true indicates the property value is to be copied.
    """
    parent: PropDesc = None
    copyFromOriginal: bool = False


@dataclass
class SaveAsDesc(TcBaseObj):
    """
    This structure captures the list of property description data for 'SaveAs' Descriptor. Clients can use this
    information to generically construct 'SaveAs' dialogs for any business object.
    
    :var businessObjectName: Name of business object that owns the descriptor
    :var propDescs: List of property description data
    """
    businessObjectName: str = ''
    propDescs: List[PropDescSaveAs] = ()


@dataclass
class SaveAsDescResponse(TcBaseObj):
    """
    Structure that contains metadata information about the properties relevant to the SaveAs operation i.e, property is
    mandatory, property is visible, if value is to be copied from original object, etc. It also has the 'DeepCopyData'
    embedded within which is a recursive data structure. The 'DeepCopyData' contains information about how the
    secondary objects (related and referenced objects) are to be copied (reference, copy as object, no copy).
    
    :var saveAsDescMap: A map of business object types and SaveAs Descriptors (string/SaveAsDesc).  This is the
    container of metadata for SaveAs that can be used by clients to generically construct SaveAs dialogs for business
    objects.
    :var deepCopyInfoMap: A map of business objects and 'DeepCopyData' (business object/'DeepCopyData'). Each business
    object from the method input will have a 'DeepCopyData' in the map. The 'DeepCopyData' object contains all the
    information about how the attached objects are to be copied (Copy as Object, Copy as Reference, NoCopy, etc.). 
    'DeepCopyData' is a recursive data structure that contains the details for the attached objects at the next level.
    NOTE: Attached objects can be either referenced objects or related objects.
    :var serviceData: Service data containing errors, etc. The plain object list of the Service data is populated with
    the target objects which are to be copied as part of the SaveAs operation. If there is an error retrieving Business
    Object for the business object name corresponding to the target object, it is added to the error stack of the
    'ServiceData' as a partial error
    """
    saveAsDescMap: SaveAsDescMap = None
    deepCopyInfoMap: DeepCopyInfoMap = None
    serviceData: ServiceData = None


@dataclass
class SaveAsInput(TcBaseObj):
    """
    This structure stores all the property input that are to be set to the object copy by the 'SaveAs' operation.
    
    :var boName: Business object name
    :var stringProps: Map (string, string) containing propName, stringValue> pairs for string properties
    :var boolArrayProps: Map (string, vector<bool>) containing <'propName, boolList'> for bool array properties
    :var dateProps: Map (string, DateTime) containing <'propName, dateTimeValue'> for date properties
    :var dateArrayProps: Map (string, vector<DateTime>) containing <'propName, dateTimeList'> for date  array properties
    :var tagProps: Map ('string, BusinessObject') containing <propName, businessObject> for reference properties
    :var tagArrayProps: Map (string, vector<BusinessObject>) containing <'propName, businessObjectList'> for reference
    array properties
    :var stringArrayProps: Map (string, vector<string>) containing <'propName, stringList'> pairs for string array
    properties
    :var doubleProps: Map (string, double) containing <'propName, doubleValue'> pairs for double properties
    :var doubleArrayProps: Map (string, vector<double>) containing <'propName, doubleList'> for double array properties
    :var floatProps: Map (string, float) containing <'propName, floatValue'> for float properties
    :var floatArrayProps: Map (string, vector<float>) containing <'propName, floatList'> for float  array properties
    :var intProps: Map (string, int) containing <'propName, intValue'> for int properties
    :var intArrayProps: Map (string, vector<int>) containing <'propName, intList'> for int array properties
    :var boolProps: Map (string, bool) containing <'propName, boolValue'> for bool properties
    """
    boName: str = ''
    stringProps: StringMap = None
    boolArrayProps: BoolVectorMap = None
    dateProps: DateMap = None
    dateArrayProps: DateVectorMap = None
    tagProps: TagMap = None
    tagArrayProps: TagVectorMap = None
    stringArrayProps: StringVectorMap = None
    doubleProps: DoubleMap = None
    doubleArrayProps: DoubleVectorMap = None
    floatProps: FloatMap = None
    floatArrayProps: FloatVectorMap = None
    intProps: IntMap = None
    intArrayProps: IntVectorMap = None
    boolProps: BoolMap = None


@dataclass
class DeepCopyData(TcBaseObj):
    """
    DeepCopyData stores the deep copy information and 'OperationInput' data of the objects that will be copied via
    'saveAs' or revise operation. It also stores the nested deep copy data at the next level.
    
    :var attachedObject: Other side object
    :var propertyName: Name of relation type or reference property for which 'DeepCopy' rule is configured
    :var propertyType: Enumeration indicating if 'DeepCopyRule' is configured for relation or feference property
    :var copyAction: DeepCopy action [NoCopy, CopyAsReference, CopyAsObject or Select]
    :var isTargetPrimary: Flag indicating if target object is primary or secondary
    :var isRequired: If flag is false, the copy action can be dynamicaly changed by user
    :var copyRelations: Flag indicating whether the properties on the relation object itself need to be copied or not
    :var saveAsInputTypeName: SaveAsInput type name
    :var childDeepCopyData: List of DeepCopy data for the objects on the other side of the relation property or
    reference property
    :var saveAsInput: Store the user inputs on the 'SaveAs' dialog. NOTE: This field is not used in the 'getSaveAsDesc'
    operation. It is used in the 'saveAsObjects' operation
    """
    attachedObject: BusinessObject = None
    propertyName: str = ''
    propertyType: PropertyType = None
    copyAction: str = ''
    isTargetPrimary: bool = False
    isRequired: bool = False
    copyRelations: bool = False
    saveAsInputTypeName: str = ''
    childDeepCopyData: List[DeepCopyData] = ()
    saveAsInput: SaveAsInput = None


class PropertyType(Enum):
    """
    This is an enumeration of the property types for which DeepCopy Rule configuration is enabled.
    """
    Relation = 'Relation'
    Reference = 'Reference'


"""
Map contains a list of <name, value> pairs '(string, DateTime').  Each pair stores information for a date property. In each pair,  name is the property name and value is  a DateTime for the property.
"""
DateMap = Dict[str, datetime]


"""
Map contains a list of <name, valueList> pairs ('string, vector< DateTime>').  Each pair stores information for a Date array property. In each pair, name is the property name and value is a list of DateTime for the property.
"""
DateVectorMap = Dict[str, List[datetime]]


"""
Map contains a list of <name, value> pairs ('BusinessObject, vector<DeepCopyData>').  For each pair, name is the business object and value holds  the deep copy data for the business object.
"""
DeepCopyInfoMap = Dict[BusinessObject, List[DeepCopyData]]


"""
Map contains a list of <name, value> pairs ('string, double').  Each pair stores information for a double property. In each pair, name is the property name and value is a double for the property.
"""
DoubleMap = Dict[str, float]


"""
Map contains a list of <name, valueList> pairs ('string, vector< double>').  Each pair stores information for a double array property. In each pair, name is the property name and value is a list of doubles for the property.
"""
DoubleVectorMap = Dict[str, List[float]]


"""
Map contains a list of <name, value> pairs ('string, float').  Each pair stores information for a float property. In each pair,  name is the property name and value is a float for the property.
"""
FloatMap = Dict[str, float]


"""
Map contains a list of <name, valueList> pairs ('string, vector< float>').  Each pair stores information for a float array property. In each pair, name is the property name and  value is a list of floats for the property.
"""
FloatVectorMap = Dict[str, List[float]]


"""
Map contains a list of <name, value> pairs '(string, vector< int>').  Each pair stores information for an int property. In each pair,  name is the property name and value is an int value for the property.
"""
IntMap = Dict[str, int]


"""
Map contains a list of <name, valueList> pairs ('string, int').  Each pair stores information for an int array property. In each pair, name is the property name and  value is the list of ints for the property.
"""
IntVectorMap = Dict[str, List[int]]


"""
Map contains a list of <name, value> pairs ('string, SaveAsDesc').  For each pair, name is the business object name and value corresponds to 'SaveAs' Descriptor data for the business object.
"""
SaveAsDescMap = Dict[str, SaveAsDesc]


"""
Map contains a list of <name, value> pairs ('string, bool').  Each pair stores information for a bool property. In each pair, name is the property name and  value is a bool for the property.
"""
BoolMap = Dict[str, bool]


"""
Map contains a list of <name, value> pairs ('string, string').  Each pair stores information for a string property. In each pair,  name is the property name and  value is a string for the property.
"""
StringMap = Dict[str, str]


"""
Map contains a list of <name, valueVector> pairs '(string, vector< string>').   Each pair stores information for a string array property. In each pair, name is the property name  and value is a list of strings for the property.
"""
StringVectorMap = Dict[str, List[str]]


"""
Map contains a list of <name, valueVector> pairs.  Each pair stores information for a bool array property. In each pair, name is the property name and value is a list of bool values for the property.
"""
BoolVectorMap = Dict[str, List[bool]]


"""
Map contains a list of <name, value> pairs ('string, BusinessObjec't).  Each pair stores information for a reference property. In each pair, name is the property name and  value is the business object for the property.
"""
TagMap = Dict[str, BusinessObject]


"""
Map contains a list of <name, valueList> pairs ('string, vector<BusinessObject>').  . Each pair stores information for a reference array property. In each pair, name is the property name and value is the list of business objects for the property.
"""
TagVectorMap = Dict[str, List[BusinessObject]]



from __future__ import annotations

from tcsoa.gen.Classification._2007_01.Classification import ClassificationPropertyValue, AttributeFormat
from tcsoa.gen.BusinessObjects import BusinessObject
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class ExtendedProeprty(TcBaseObj):
    """
    This Structure represents extended metadata property.
    
    :var propertyName: Name of the extended metadata property associated to the given attribute.
    :var values: List of references to the 'ExtendedPropertyValues' structure holding actual values of the Extended
    metadata properties.
    """
    propertyName: str = ''
    values: List[ExtendedPropertyValues] = ()


@dataclass
class ExtendedPropertyValues(TcBaseObj):
    """
    This structure is used to hold extended metadata property values
    
    :var value: Values of the Extended metadata property associated to a Classification Attribute.
    """
    value: str = ''


@dataclass
class FormatProperties(TcBaseObj):
    """
    Structure representing format details
    
    :var format: References the 'AttributeFormat' structure holding the format definition for this attribute.
    :var unitName: Unit display name associated with this attribute in this unit system.
    :var defaultValue: Default value of this Class attribute. This can be an empty string indicating no default value.
    :var minimumValue: Minimum value constraint of this Class attribute. This can be an empty string indicating no
    minimum value constraint.
    Note: Only applicable to numerical formats of attributes
    :var maximumValue: Maximum value contraint of this Class attribute. This can be an empty string indicating no
    maximum value constraint.
    Note: Only applicable to numerical formats of attributes
    """
    format: AttributeFormat = None
    unitName: str = ''
    defaultValue: str = ''
    minimumValue: str = ''
    maximumValue: str = ''


@dataclass
class GetAttributesForClassesResponse2(TcBaseObj):
    """
    Holds the values returned by 'getAttributesForClass2()'
    
    :var attributes: Map of Class IDs and the Class attributes found for each of those classes.
    :var data: Any failure will be returned with Class ID mapped to the error message in the ServiceData list of
    partial errors.
    """
    attributes: StrVClsAttrMap2 = None
    data: ServiceData = None


@dataclass
class GetKeyLOVsResponse2(TcBaseObj):
    """
    Holds the keylovs returned by the' ''getKeyLOVs2()' method.
    
    :var keyLOVs: Map of KeyLOV definitions details.
    :var data: Any failure will be returned with Key-LOV ID mapped to the error message in the 'ServiceData' list of
    partial errors
    """
    keyLOVs: StrKeyLOVDefMap2 = None
    data: ServiceData = None


@dataclass
class KeyLOVDefinition2(TcBaseObj):
    """
    Structure representing KeyLOVDefinition
    
    :var id: Unique Key-LOV ID. This is a negative number.This can be found in the Key/ID field of a Key-LOV definition
    in the Key-LOV tab of Classification Administration.
    :var name: Name of the Key-LOV.
    :var keyLovtag: Unused parameter. Reserved for future use
    :var options: Key-LOV options to Show/Hide keys.Valid values are: 
    - 0 = Show key
    - 1 = Hide key
    
    
    :var keyLovEntries: List of Key-LOV entries.
    :var owningSite: Owning Site (POM_imc).
    :var sharedSites: List of sites (POM_imc) where this Key-LOV is to be shared using Multisite operations.
    """
    id: str = ''
    name: str = ''
    keyLovtag: BusinessObject = None
    options: int = 0
    keyLovEntries: List[KeyLOVEntry] = ()
    owningSite: BusinessObject = None
    sharedSites: List[BusinessObject] = ()


@dataclass
class KeyLOVEntry(TcBaseObj):
    """
    Structure representing KeyLOVEntry
    
    :var lovKey: String representing a Key of a Key-LOV entry.
    :var lovValue: String representing a Value  of the Key-LOV  entry.
    :var deprecatedSatus: Flag indicating whether this particular Key-LOV entry is deprecated or not.
    """
    lovKey: str = ''
    lovValue: str = ''
    deprecatedSatus: bool = False


@dataclass
class AttributeConfiguration(TcBaseObj):
    """
    Holds the configuration attached to an attibute
    
    :var preConfig: Preconfiguration attached to the attribute
    :var config: Base configuration attached to the attribute.
    :var postConfig: Post configuration attached to the attribute.Configurations could be any combinations of the
    following individual configurations:
    - multifield
    - horizontal
    - vertical
    - separator
    - arrow
    - button
    - wide
    - mandatory flag
    - protected flag
    - unique flag
    
    """
    preConfig: str = ''
    config: str = ''
    postConfig: str = ''


@dataclass
class AutoComputeAttribute(TcBaseObj):
    """
    Contains the attributes that are used for the auto compute operation.
    
    :var values: List of references to the 'ClassificationPropertyValue' structures that will hold the values (single
    or multiple values) for the Attribute represented by this 'AutoComputeAttribute'.
    :var unitName: Display unit name for this attribute.
    :var length: Length of an Attribute. This field will hold the number of values for this Attribute. 
    - For a single valued attribute, length = 1
    - For an array type of attribute, length = n [where n is the size of the array.]
    
    
    :var attributeProperties: Represents the Attribute property flags concatenated into a single Integer value.
    To access  individual flag, a bitwise operation will need to be performed by the Client.
    Valid values are as specified in the 'enum AutoComputeAttrPropertyFlags'.
    :var isModified: Flag to indicate whether the attribute referenced by this 'AutoComputerAttribute' structure is
    modified by the client. It is only used when 'AutoComputeAttribute' structure is used as an Input parameter to the
    'autoComputeAttributes' operation. This parameter is should not be used by clients when reading the
    'AutoComputeAttribute' structure as part of the  returned 'AutoComputeAttributesResponse'.
    """
    values: List[ClassificationPropertyValue] = ()
    unitName: str = ''
    length: int = 0
    attributeProperties: int = 0
    isModified: bool = False


@dataclass
class AutoComputeAttributesResponse(TcBaseObj):
    """
    Contains the attribute values for the corresponding autocomputed attributes after the autocompute operation is
    called. The structure element is a key,value pair of attribute IDs and their corresponding attribute properties.
    The attribute properties of type AutoComputeAttribute  contains the following information:
    - values : This structure element of type ClassificationPropertyValue holds the classification properties for the
    classification object
    - length : the number of attributes computed
    - unitName : the unit of measure for the attribute
    - attributeProperties : Properties of the attributes
    - isModified : Boolean value to check if attribute is modified
    
    
    
    :var autoComputedAttrs: Map containing references of the 'AutoComputeAttribute' structures with their respective
    attribute IDs.
    :var serviceData: Any failures with classification object ID mapped to the error message in the ServiceData list of
    partial errors.
    """
    autoComputedAttrs: AutoComputeAttributesMap = None
    serviceData: ServiceData = None


@dataclass
class ClassAttribute2(TcBaseObj):
    """
    Structure representing class attribute details.
    
    :var id: ID for this attribute.
    :var name: Name for this attribute.
    :var extendedProperties: List of references to the ExtendedProeprty structure holding the extended metadata
    properties of this attribute.
    :var shortName: Short name defined for this attribute.
    :var description: Description added for this attribute.
    :var metricFormat: References the 'FormatProperties' structure defining the metric format of this attribute.
    :var nonMetricFormat: References the 'FormatProperties' structure defining the nonmetric format of this attribute.
    :var annotation: Annotation information added to this attribute.
    :var attributeConfiguration: Reference the 'AttributeConfiguration' structure that defines any additional
    configuration for this attribute.
    :var arraySize: Array size or the number of values for this attribute.
    - If single valued (nonarray), then 'arraySize' = 1
    - If multi valued (array), then 'arraySize' >= 1 corresponding to the size of the array defined in the attribute
    definition.
    
    
    :var options: Attribute property flags represented as a single integer. To access individual property flags, a
    bitwise operation will be required by the client.Valid values are:
    - ATTR_vla                      = (1 << 0)
    - ATTR_external_vla      = (1 << 1)
    - ATTR_mandatory         = (1 << 2)
    - ATTR_protected             = (1 << 3)
    - ATTR_unique                 = (1 << 4)
    - ATTR_propagated         = (1 << 5)
    - ATTR_localValue             = (1 << 6)
    - ATTR_reference             = (1 << 7)
    - ATTR_auto_computed = (1 << 15)
    - ATTR_hidden                 = (1 << 20)
    - ATTR_localizable         = (1 
    
    """
    id: int = 0
    name: str = ''
    extendedProperties: List[ExtendedProeprty] = ()
    shortName: str = ''
    description: str = ''
    metricFormat: FormatProperties = None
    nonMetricFormat: FormatProperties = None
    annotation: str = ''
    attributeConfiguration: AttributeConfiguration = None
    arraySize: int = 0
    options: int = 0


"""
Structure elements:

- Primary key - Integer Attribute IDs for attributes. This will be provided as an input. 
- Values - References an AutoComputeAttribute structure  for the corresponding attribute ID in the primary key of this map.


"""
AutoComputeAttributesMap = Dict[int, AutoComputeAttribute]


"""
Map to store Key-LOV definitions details.Any failure will be returned with the Key-LOV ID mapped to the error message in the 'ServiceData' list of partial errors.
"""
StrKeyLOVDefMap2 = Dict[str, KeyLOVDefinition2]


"""
StrVClsAttrMap2
"""
StrVClsAttrMap2 = Dict[str, List[ClassAttribute2]]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import WorkspaceObject, POM_object, Awb0ProductContextInfo
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass
from typing import Dict, List


@dataclass
class UpdateContentBasedOnRevInput(TcBaseObj):
    """
    This structure contains WorkspaceObject and Awb0ProductContextInfo needed to reconfigure and update the content for
    impacted occurrences of the WorkspaceObject.
    
    :var workspaceObject: WorkspaceObject whose occurrences in content needs to be reconfigured. Supported type :
    ItemRevision
    :var productContext: The Awb0ProductContextInfo object containing configuration information.
    :var requestPref: A map (string, list of string) of preference names and values. Preference names and values are
    case sensitive.
    """
    workspaceObject: WorkspaceObject = None
    productContext: Awb0ProductContextInfo = None
    requestPref: RequestPreference11 = None


@dataclass
class UpdateWorkingContextInput(TcBaseObj):
    """
    UpdateWorkingContextInput contains working context container on which configuration and recipe to be saved.
    
    :var productInfo: Awb0ProductContextInfo containing the product and configuration.
    :var workingContext: Object to which configuration and recipe to be saved. Valid business object types are:
    Fnd0AppSession.
    :var saveResult: If true, persist results on Fnd0StructureContextData; otherwise, results will not persisted on
    Fnd0StructureContextData.
    :var operation: The operation to be performed. Valid values  are: "Create", "SaveAs"
    Create : Indicates user have performed create operation
    SaveAs : Indicates user have performed SaveAs operation
    """
    productInfo: Awb0ProductContextInfo = None
    workingContext: POM_object = None
    saveResult: bool = False
    operation: str = ''


"""
The map which can have a key and value pair, used for occurrences expansion filters or criteria or options. The key and value are case sensitive.
"""
RequestPreference11 = Dict[str, List[str]]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import POM_object, ConfigurationFamily, Variant
from enum import Enum
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class TeamcenterConfiguratorInfo(TcBaseObj):
    """
    Contains additional information about variant data that is only available when using Teamcenter configurators. For
    any non Teamcenter configurator, this structure may contain null elements.
    The 'TeamcenterConfiguratorInfo' structure is interoperated as "NULL" if it has following values set for its
    parameters:
    variant : NULLTAG
    optionValue : NULLTAG
    
    
    :var variant: Teamcenter family business object of type Variant. It can be null if information is retrieved from
    non Teamcenter configurator.
    :var optionValue: Workspace representation of option value code. This parameter can be null if information is being
    retrieved from non Teamcenter configurator.
    """
    variant: Variant = None
    optionValue: POM_object = None


@dataclass
class VariantCriteria(TcBaseObj):
    """
    The 'VariantCriteria' structure represents a criteria object along with its variant criteria and associated
    validation records. The 'VariantCriteria' structure is interpreted to be "NULL" if the following values are set for
    its parameters:
    - 'variantCriteriaObject': NULLTAG
    - 'primaryVariantScope': a "NULL" ConfigFormula structure
    - 'solveType': -1
    - 'validationRecords': empty vector
    
    :var variantCriteriaObject: The VariantRule with which the criteria defines in this structure is associated.
    :var primaryVariantScope: Nominal variant criteria before applying constraints and defaults.
    :var solveType: Indicates how the variant criteria need to be solved. For possible values see preference
    'TC_Default_Solve_Type'.
    :var validationRecords: Validation records related to the variant criteria.
    """
    variantCriteriaObject: POM_object = None
    primaryVariantScope: ConfigExpression = None
    solveType: int = 0
    validationRecords: List[VariantValidationRecord] = ()


@dataclass
class VariantCriteriaResponse(TcBaseObj):
    """
    Returns variant configuration criteria along with default and constraint processing results.
    
    :var variantCriteria: Vector of variant criteria structures
    :var serviceData: Teamcenter ServiceData
    """
    variantCriteria: List[VariantCriteria] = ()
    serviceData: ServiceData = None


@dataclass
class VariantValidationRecord(TcBaseObj):
    """
    Represents validation results for variant criteria associated with a variant criteria object (VariantRule ).
    The 'VariantValidationRecord' structure is interpreted to be "NULL" if the following values are set to its
    parameters:
    
    '-validationDate': a NULLDATE as defined for the SOA client side binding in use.
    
    '-validationResult': a "NULL" ConfigFormula structure
    
    '-undeterminedFamilies': an empty vector
    
    '-violations': an empty vector
    
    '-appliedDefaults': an empty vector
    
    :var validationDate: Date of validation.
    :var validationResult: Validation result reflecting constraints and defaults.
    :var undeterminedFamilies: Families that are not set to a unique value. If empty, the criteria are complete.
    :var violations: Violation records. If empty, the variant criteria are valid.
    :var appliedDefaults: One record for each applied default in the sequence they were applied.
    """
    validationDate: datetime = None
    validationResult: ConfigExpression = None
    undeterminedFamilies: List[ConfigurationFamily] = ()
    violations: List[ConfigRuleViolation] = ()
    appliedDefaults: List[DefaultRule] = ()


@dataclass
class ConfigExpression(TcBaseObj):
    """
    Represents a configuration expression. The 'ConfigExpression' structure is interpreted as "'NULL'" when it has
    following values set for its parameters:
    'subExpressions' : An empty vector of 'ConfigExpression' structure
    'value' : A "NULL" 'ConfigOption' structure
    'opCode' : -1
    'formula' : A "NULL" 'ConfigFormula' structure
    'tnf' : An empty vector of 'ConfixExpression' structures.
    A 'ConfigExpression' that is equivalent to the Boolean constant 'TRUE' returns a TNF parameter comprising one
    expression with an 'opcode' value of '39'.
    A 'ConfigExpression' that is equivalent to the Boolean constant 'FALSE' returns a TNF parameter comprising one
    expression with an 'opcode' value of '40'.
    
    :var subExpressions: A vector of 'ConfigExpression'. If this parameter is present the enclosing expression
    represents a compound expression combining this set of subexpressions with the operator specified in the opCode
    parameter. If the enclosing expression is non-NULL either this parameter or the "'value'" parameter must be present.
    :var value: It is a 'ConfigOption' struct. If this parameter is present the enclosing expression represents an
    elemental expression literal. The parameter specifies the value to which 'opCode' compares. If the enclosing
    expression is non-NULL either this parameter or the 'subExpressions' parameter must be present. 'subExpressions'
    'and' value must not both be present.
    :var opCode: Operation code such as "and" and "or", see ps/ps_tokens.h.
    :var formula: It is a 'ConfigFormula' struct. If present, contains the configuration expression in string format
    (courtesy info).
    :var tnf: If present, this parameter provides a representation of the enclosing expression in Teamcenter Normal
    Form (TNF).
    The parameter is intended as "courtesy information" so to make additional SOA calls unnecessary by providing
    information that is expected to be of value if performance permits it. Based on the assumption that the vast
    majority of ConfigExpression structs can be converted to TNF very efficiently the server can afford to convert them
    to TNF as a "free gift". If this becomes a performance bottleneck the server may return a response with an empty
    tnf member, in which case the application can explicitly request a tnf representation using operation
    'convertVariantExpressions', if need be. Preference 'TC_tnf_timeout_period' controls the timeout mechanism. If TNF
    generation takes more time than specified in preference 'TC_tnf_timeout_period', server returns an empty 'tnf'
    member.
    If combined with OR the list of tnf expressions is logically equivalent to the enclosing expression. Each TNF is
    provided as a conjunction of Disjunctive Normal Forms (DNF) where all DNFs reference a single variant option
    family. Each clause in the outermost conjunction can reference a different variant option family.
    Example 1:
    Formula: (ENG = V6 | ENG = V8) & (D = 2.6L | D >= 3L & D <= 3.6L)
    TNF: Yes
    Note: DNF1 references family ENG and DNF2 references family D
    Example 2:
    Formula: (ENG = V6 | TRANS = AUTO) & (TRANS = M5 | ENG = V8)
    TNF: No
    Note: DNF1 references family ENG as well as TRANS and  DNF2 references family ENG as well as TRANS
    """
    subExpressions: List[ConfigExpression] = ()
    value: ConfigOption = None
    opCode: int = 0
    formula: ConfigFormula = None
    tnf: List[ConfigExpression] = ()


@dataclass
class ConfigFormula(TcBaseObj):
    """
    Represents a formula string associated with a configuration expression in configurator syntax. 
    A formula of the form "[OptionNamespace]FamilyName = UniqueValue" is called "explicit" because no product context
    is required to determine the family name for the value, or the option namespace for the family name.
    OptionNamespace and FamilyName are explicitly spelled out. A variant formula is in "explicit Teamcenter language"
    if its form is explicit and comprised of the lexemes documented for the Variant Formula property.
    A formula of the form "UniqueValue" is called "stenographic" because a parser has to infer OptionNamespace and
    FamilyName. This is possible if there is only one value with that name throughout the entire product.
    If lexemes other than those documented for the Variant Formula property are used the formula is in "external
    variant language" and a custom configurator is required to decode the formula.
    If a formula is explicit Teamcenter language, productName and productNameSpace can be empty ("").
    The ConfigFormula structure is considered "NULL" if it has following values set for its parameters:
    - formula: empty string ("")
    - productName: empty string ("")
    - productNameSpace: empty string ("")
    
    :var formula: Formula string in configurator syntax.
    :var productName: Name of the product(e.g.ItemID). Used in conjunction with productNameSpace parameter to resolve
    any ambiguity in variant option value names.
    :var productNameSpace: Namespace of the product in which the "product" has unique semantics, e.g., ItemRevID,
    model_year or product_type. Used in conjunction with productName parameter to resolve any ambiguity in variant
    option value names.
    """
    formula: str = ''
    productName: str = ''
    productNameSpace: str = ''


@dataclass
class ConfigOption(TcBaseObj):
    """
    Represents a configuration option that is unique in the context of a ConfigurationFamily. 
    The ConfigOption structure is interpreted as "NULL" if it has the following values set for its parameters: 
    family: NULLTAG
    valueCode: empty string ("")
    description: empty string ("")
    additionalInfo : Null Structure
    
    
    :var family: A ConfigurationFamily object which represents a configuration option.
    :var valueCode: Name of the option value. "" is treated as NO VALUE SELECTED, i.e. it does not match any non-empty
    name.
    :var description: Description of the option value.
    :var additionalInfo: A 'TeamcenterConfiguratorInfo' struct having additional information about Teamcenter
    configurator objects.
    """
    family: ConfigurationFamily = None
    valueCode: str = ''
    description: str = ''
    additionalInfo: TeamcenterConfiguratorInfo = None


@dataclass
class ConfigRuleViolation(TcBaseObj):
    """
    Represents a violated rule along with the message and severity associated with rule violation.
    
    The 'ConfigRuleViolation' structure is interpreted to be "NULL" if following values are set to its parameters:
    
    - 'message': empty string ("")
    
    - 'severity': ConstraintSeverityInformation
    
    - 'violatedCondition': a "NULL" 'ConfigExpression' structure
    
    :var message: The message to display if this rule is violated.
    :var severity: The severity code for the message. Valid values are:
    
    - ConstraintSeverityInformation : Classifies information associated with this constraint as additional information,
    such as hints, which are of interest if configuration criteria satisfy this constraint.
    
    - ConstraintSeverityWarning : Classifies information associated with this constraint as considerations, such as
    recommendations, which are important to review if configuration criteria satisfy this constraint.
    
    - ConstraintSeverityError : Configuration criteria that satisfy this constraint are classified as invalid.
    :var violatedCondition: The rule that was found to be violated.
    """
    message: str = ''
    severity: ConstraintSeverity = None
    violatedCondition: ConfigExpression = None


@dataclass
class DefaultRule(TcBaseObj):
    """
    A rule to determine the default value, or value range, for a variant option family. 
    The 'DefaultRule' structure is interpreted to be "NULL" if it has following values set to its parameters:
    
    - 'partiallyApplicable': false
    
    - 'restrictiveCondition': a "NULL" ConfigExpression
    
    - 'appliedDefault': "NULL" ConfigExpression structure
    
    - 'appliedTo': "NULL" ConfigExpression structure
    
    :var partiallyApplicable: If "true", the default is only applicable to a subset of the criteria (courtesy info).
    :var restrictiveCondition: The condition under which the default is applicable. Can be NULL.
    :var appliedDefault: The default condition that was applied, indicating that "'restrictiveCondition'" was met.
    :var appliedTo: The config condition before applying the default."'appliedDefault'".
    """
    partiallyApplicable: bool = False
    restrictiveCondition: ConfigExpression = None
    appliedDefault: ConfigExpression = None
    appliedTo: ConfigExpression = None


class ConstraintSeverity(Enum):
    """
    Severity code for error message.
    
    :var ConstraintSeverityInformation: Classifies information associated with this constraint as additional
    information, such as hints, which are of interest if configuration criteria satisfy this constraint.
    :var ConstraintSeverityWarning: Classifies information associated with this constraint as considerations, such as
    recommendations, which are important to review if configuration criteria satisfy this constraint.
    :var ConstraintSeverityError: Configuration criteria that satisfy this constraint are classified as invalid.
    """
    Information = 'ConstraintSeverityInformation'
    Warning = 'ConstraintSeverityWarning'
    Error = 'ConstraintSeverityError'



from __future__ import annotations

from typing import List
from tcsoa.gen.Manufacturing._2015_03.StructureManagement import MoveAndResequenceParameter, MoveAndResequenceResponse
from tcsoa.utils import TcService


class StructureManagementService(TcService):

    @classmethod
    def moveAndResequenceNodes(cls, inputList: List[MoveAndResequenceParameter]) -> MoveAndResequenceResponse:
        """
        This operation moves and/or re-sequences the nodes in the structure.
        
        Use cases:
        Use Case 1: Re-sequencin g nodes within parent in a structure.
        This operation can be used in M anufacturing Process Planning (MPP) application to resequence a node in a
        struct ure. For example, in a product structure an Item or  BOMLine can be reseqenced within the parent by
        dragging and dropping t he Item in between the siblings. Same could be done in proces s structure by dragging
        and dropping a process or operation BOMLine. As a result of drop, the find number of the node and its
        subsequent sibling are modified.
        Use Case 2: Re-parent and re-sequence of nodes.
        This operation can be used in MPP application to move the nodes from one parent to another and sequenced them
        among the siblings of new parent. For example, in a process structure a process or operation BOMLine objects
        can be dragged and dropped in between the child nodes of another process BOMLine. The dragged processes or
        operations are moved as children of the new process BOMLine and sequenced among the sibling as per dropped
        location. While drag and drop, UI presents an option whether to clone the nodes. If selected then the nodes are
        cloned instead of re-parent.
        Use Case 3: Moving the processes or operations to process resource in Plant Bill of Processes (BOP).
        This operation can be used to allocate the process or operation BOMLine of type Mfg0BvrProcess or
        Mfg0BvrOperation to a process resource of type Mfg0BvrProcessResource in a Plant BOP structure. The find number
        of dropped BOMLine objects could be calculated either in the context of the process station of type
        Mfg0BvrProcessStation to which process resource is a child object, or process resource is itself. If the
        context is process station of type Mfg0BvrProcessStation then the find numbers of the dropped BOMLine objects
        are calculated based on the existing processes or operations that are a child object of process station. If the
        context is process resource then the calculation is based on the processes or operations allocated to that
        process resource. In Teamcenter MPP application, the context is always a process station BOMLine.
        """
        return cls.execute_soa_method(
            method_name='moveAndResequenceNodes',
            library='Manufacturing',
            service_date='2015_03',
            service_name='StructureManagement',
            params={'inputList': inputList},
            response_cls=MoveAndResequenceResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, FND_TraceLink
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class CreateInput(TcBaseObj):
    """
    CreateInput structure used to capture the inputs required for creation of a business object. This is a recursive
    structure containing the CreateInput(s) for any secondary(compounded) objects that might be created (e.g Item also
    creates ItemRevision and ItemMasterForm etc.)
    
    :var boName: The type name of the Business Object.
    :var propertyNameValues: Map of property name (key) and property values (values) in string format, to be set on new
    object being created. The calling client is responsible for converting the different property types (int, float,
    date .etc) to a string using the appropriate function(s) in the SOA client framework Property class.
    :var compoundCreateInput: CreateInput structure used to capture the inputs required for creation of a business
    object. This is a recursive structure containing the CreateInput(s) for any secondary(compounded) objects that
    might be created (e.g Item also creates ItemRevision and ItemMasterForm etc.)
    """
    boName: str = ''
    propertyNameValues: PropertyValues = None
    compoundCreateInput: CreateInputMap = None


@dataclass
class CreateTracelinksInputData(TcBaseObj):
    """
    Structure represents the parameters required to create a tracelink between the given set of objects.
    
    :var clientId: A unique string supplied by the caller. This ID is used to identify return data elements and partial
    errors associated with this input structure.
    :var tracelinkCreateInput: The Create Input structure which holds the necessary information to create an object of
    type FND_TraceLink or its subtypes.
    :var primaryObj: The business object which represents the Defining end of the tracelink. If it is Occurrence,
    FND_TraceLink object will be created with its underlying ItemRevision or AbsoluteOccurrence object depending on the
    input preference.
    :var secondaryObj: The business object which represents the Complying end of the tracelink. If it is Occurrence,
    FND_TraceLink object will be created with its underlying ItemRevision or AbsoluteOccurrence object depending on the
    input preference.
    :var requestPref: A map of preferences and their values (string/string). For e.g. keys could be
    createComplyingTracelinkWithOccurrence, createDefiningTracelinkWithOccurrence, createTracelinkWithOccurrences etc.
    and values could be true or false and are case sensitive.
    """
    clientId: str = ''
    tracelinkCreateInput: CreateInput = None
    primaryObj: BusinessObject = None
    secondaryObj: BusinessObject = None
    requestPref: RequestPreference2 = None


@dataclass
class CreateTracelinksResponse(TcBaseObj):
    """
    Structure represents the output parameters of the operation.
    
    :var output: A list of CreateTracelinksResponseData structures.
    :var serviceData: The Service Data.
    """
    output: List[CreateTracelinksResponseData] = ()
    serviceData: ServiceData = None


@dataclass
class CreateTracelinksResponseData(TcBaseObj):
    """
    Structure represents the output of the operation.
    
    :var clientId: The clientId from the input CreateTracelinksInputData element. This value is unchanged from the
    input, and can be used to identify this response element with the corresponding input element.
    :var tracelinkObject: The FND_TraceLink object.
    """
    clientId: str = ''
    tracelinkObject: FND_TraceLink = None


"""
PropertyValues is a map of property name (key) and property values (values) in string format.
"""
PropertyValues = Dict[str, List[str]]


"""
A map of preferences and their values. Current supported key values are createTracelinkWithOccurrences, createComplyingTracelinkWithOccurrence, createDefiningTracelinkWithOccurrence. The supported values are "true" and "false". Keys and their values both are case sensitive. 
If createComplyingTracelinkWithOccurrence is "true", the complying object for the created tracelink will be the occurrence.
If createDefiningTracelinkWithOccurrence is "true", the defining object for the created tracelink will be the occurrence. 
If createTracelinkWithOccurrences is "true", the defining and complying object for the created tracelink will be the occurrence.
"""
RequestPreference2 = Dict[str, str]


"""
CreateInputMap is a map of reference or relation property name to secondary CreateInput objects.
"""
CreateInputMap = Dict[str, List[CreateInput]]



from tcsoa.gen.ActiveCollaboration._2020_12.services import ActiveCollaborationService as imp0
from tcsoa.utils import TcService


class ActiveCollaborationService(TcService):
    createOrUpdateComment = imp0.createOrUpdateComment
    createOrUpdateConversation = imp0.createOrUpdateConversation
    deleteComment = imp0.deleteComment
    deleteConversation = imp0.deleteConversation
    getReadInfo = imp0.getReadInfo
    manageSubscriptions = imp0.manageSubscriptions



from __future__ import annotations

from tcsoa.gen.BusinessObjects import GeneralDesignElementLink, GeneralDesignElement, GDEbvr, Form, GDEOccurrence, RevisionRule, ItemRevision
from typing import Dict, List
from tcsoa.gen.Cad._2007_01.StructureManagement import OccNote
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class FeatureQueryData(TcBaseObj):
    """
    Contains processing information for 'queryRelatedFeatures' operation.
    
    :var itemRevision: Item revision, starting point for retrieving related features.
    :var occNote: Occurrence data for selection of the featureTypes to process, if null then all will be processed.
    :var featureTypes: Types of features related to the ItemRevision to process, if empty then all will be processed.
    :var gdeOccurrence: Feature occurrence, starting point for retrieving related features. If supplied, the above data
    is ignored.
    :var gdeBVR: The GDE BOM View Revision for the Item Revision.
    """
    itemRevision: ItemRevision = None
    occNote: OccNote = None
    featureTypes: List[str] = ()
    gdeOccurrence: GDEOccurrence = None
    gdeBVR: GDEbvr = None


@dataclass
class FeatureQueryInfo(TcBaseObj):
    """
    Contains 'clientId', 'featureQueryData' and 'featureReturnFilter' information.
    
    :var clientID: Unique string to identify the input data, required input.
    :var featureQueryData: Structure contains the input data for querying the features.
    :var featureReturnFilter: Structure contains the filter criteria.
    """
    clientID: str = ''
    featureQueryData: FeatureQueryData = None
    featureReturnFilter: FeatureReturnFilter = None


@dataclass
class FeatureReturnFilter(TcBaseObj):
    """
    Contains filter information for queryRelatedFeatures operation.
    
    :var returnFeatureForms: Flag to specify if the Feature forms for the GDE and GDELinks returned are to be returned
    also.
    :var relatedItems: Flag to specify if the related ItemRevisions are to be returned.
    :var returnGDEs: Flag to specify if the GDEs that pass the filter for ItemRevisions returned are to be returned
    also.
    :var returnGDELinks: Flag to specify if the GDELinks that pass the filter for ItemRevisions returned are to be
    returned also.
    :var recursive: Flag to specify if the reference and referenced GDEs are to be returned( only used if relatedItems
    flag is set to true ).
    :var featureTypes: Types of features to be processed, if empty then all.
    :var configRule: Name of a configuration rule to use, if none specified then user's default is used.
    """
    returnFeatureForms: bool = False
    relatedItems: bool = False
    returnGDEs: bool = False
    returnGDELinks: bool = False
    recursive: bool = False
    featureTypes: List[str] = ()
    configRule: RevisionRule = None


@dataclass
class GDEItem(TcBaseObj):
    """
    Contains gdelink, gde and feature form information.
    
    :var gde: The GDE referenced by the Item Revision.
    :var gdeLinks: GDELinks referencing the GDE if requested.
    :var featureForm: The Feature form for the GDE if requested
    """
    gde: GeneralDesignElement = None
    gdeLinks: List[RefedGDELinkItem] = ()
    featureForm: Form = None


@dataclass
class GDELinkItem(TcBaseObj):
    """
    Contains gdelink, gde and feature form information.
    
    :var gdeLink: The GDELink referenced by the Item Revision.
    :var gdes: GDEs referenced by the GDELink if requested.
    :var featureForm: The Feature form for the GDELink if requested
    """
    gdeLink: GeneralDesignElementLink = None
    gdes: List[RefedGDEItem] = ()
    featureForm: Form = None


@dataclass
class QueryRelatedFeaturesOutput(TcBaseObj):
    """
    Contains the output data for queryRelatedFeatures operation, which contains itemrevision, gdeItem and gdeLink item.
    
    :var itemRevision: The ItemRevision of the feature for which query is being made.
    :var gdeItems: GDE items owned by this Item Revision if requested.
    :var gdeLinkItems: GDELinks owned by this Item Revision if requested.
    """
    itemRevision: ItemRevision = None
    gdeItems: List[GDEItem] = ()
    gdeLinkItems: List[GDELinkItem] = ()


@dataclass
class QueryRelatedFeaturesResponse(TcBaseObj):
    """
    Contains the response structure for queryRelatedFeatures
    
    :var returnQueryRelatedFeature: returnQueryRelatedFeature
    :var serviceData: serviceData
    """
    returnQueryRelatedFeature: ClientIdToFeatureMap = None
    serviceData: ServiceData = None


@dataclass
class RefedGDEItem(TcBaseObj):
    """
    Contains item revision, gde and feature form information for queryRelatedFeatures operation.
    
    :var itemRevision: itemRevision
    :var gde: gde
    :var featureForm: featureForm
    :var gdeLinks: gdeLinks
    """
    itemRevision: ItemRevision = None
    gde: GeneralDesignElement = None
    featureForm: Form = None
    gdeLinks: List[RefedGDELinkItem] = ()


@dataclass
class RefedGDELinkItem(TcBaseObj):
    """
    Contains item revision, gde, gdeLinks and feature form information for queryRelatedFeatures operation.
    
    :var itemRevision: The ItemRevision that owns the GDELink.
    :var gdeLink: The GDELink referenced by the ItemRevision.
    :var featureForm: The FeatureForm for the GDELink if requested
    :var gdes: GDEs referenced by the GDELink if recursive.
    """
    itemRevision: ItemRevision = None
    gdeLink: GeneralDesignElementLink = None
    featureForm: Form = None
    gdes: List[RefedGDEItem] = ()


"""
Map of input client id to feature values ('QueryRelatedFeaturesOutput').
"""
ClientIdToFeatureMap = Dict[str, QueryRelatedFeaturesOutput]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import Dataset, BOMLine, ImanFile
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetOccKinematicsInfoInput(TcBaseObj):
    """
    Input containing context and scope for which the occurrence kinematics information of resource occurrence
    Mfg0MEResourceRevision or ItemRevision is required
    
    :var context: Root BOMLine in Bill of Equipment structure representing Mfg0BvrWorkarea. The operation processes
    scope within the context.
    :var scope: BOMLine representing Mfg0BvrWorkarea object under context. The operation processes resource occurrence
    under the scope to get occurrence kinematics information. The scope can be empty, in this case context is treated
    as scope.
    """
    context: BOMLine = None
    scope: BOMLine = None


@dataclass
class GetOccurrenceKinematicsInfoResponse(TcBaseObj):
    """
    Response for getOccurrenceKinematicsInformation operation.
    
    :var occurrenceKinematicInfo: A list containing information about BOMLine representing occurrence of
    Mfg0MEResourceRevision or ItemRevision and ImanFile having occurrence kinematics information
    :var serviceData: The Teamcenter Services structure used to return status of the operation.
    """
    occurrenceKinematicInfo: List[OccurrenceKinematicsInfo] = ()
    serviceData: ServiceData = None


@dataclass
class OccurrenceKinematicsInfo(TcBaseObj):
    """
    Response containing information about context BOMLine representing Mfg0BvrWorkarea, scope BOMLine representing
    Mfg0BvrWorkarea , a map with key as BOMLine representing occurrence of Mfg0MEResourceRevision or ItemRevision and
    the value as ImanFile having occurrence kinematics information.
    
    :var context: Root BOMLine in Bill of Equipment structure representing Mfg0BvrWorkarea
    :var scope: BOMLine representing Mfg0BvrWorkarea object under context. The operation processes resource occurrence
    under the scope to get occurrence kinematics information
    :var resourceLineInformation: A map (BOMLine, ImanFile) containing information about BOMLine representing the
    occurrence of MFg0MEResourceRevision or ItemRevision as key and corresponding ImanFile as value which contains the
    kinematics information
    """
    context: BOMLine = None
    scope: BOMLine = None
    resourceLineInformation: ResourceLineInfoMap = None


"""
The map (BOMLine, Dataset) containing BOMLine representing occurrence of Mfg0MEResourceRevision or ItemRevision as key and Mfg0OccKinematicsInfo dataset as value.
"""
OccKinematicsInfoMap = Dict[BOMLine, Dataset]


"""
A map (BOMLine, ImanFile) containing information about the BOMLine representing occurrence of MFg0MEResourceRevision or ItemRevision as key and corresponding ImanFile as value which contains the kinematics information.
"""
ResourceLineInfoMap = Dict[BOMLine, ImanFile]



from __future__ import annotations

from tcsoa.gen.Internal.Cad._2008_06.DataManagement import FeatureQueryInfo, QueryRelatedFeaturesResponse
from typing import List
from tcsoa.utils import TcService


class DataManagementService(TcService):

    @classmethod
    def queryRelatedFeatures(cls, featureQueryInfos: List[FeatureQueryInfo]) -> QueryRelatedFeaturesResponse:
        """
        QueryRelatedFeatures allows the user to query for all Feature elements ( GDE and GDELink ) related to an Item
        Revision.
        This query can return all GDE and GDELink elements related to the Item Revision, the GDELinks ( and parent Item
        Revision ) that reference the GDEs and the GDEs ( and parent Item Revision ) referenced by the GDELinks.
        
        Use cases:
        The client wants to know what Feature relations exist in Teamcenter for a particular item Revision. The client
        supplies an item revision, GDE Occurrence, or Occurrence note as the starting point and specifies the data to
        be returned. The client will then get a list of GDE and GDELinks referenced in the hierarchy.
        """
        return cls.execute_soa_method(
            method_name='queryRelatedFeatures',
            library='Internal-Cad',
            service_date='2008_06',
            service_name='DataManagement',
            params={'featureQueryInfos': featureQueryInfos},
            response_cls=QueryRelatedFeaturesResponse,
        )



from __future__ import annotations

from tcsoa.gen.ProjectManagement._2009_10.ScheduleManagement import GenericAttributesContainer, GenericResponseContainer, ScheduleModifyContainer
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class ScheduleManagementService(TcService):

    @classmethod
    def modifySchedules(cls, scheduleModifyContainers: List[ScheduleModifyContainer]) -> ServiceData:
        """
        Not Implemented
        
        Exceptions:
        >None
        """
        return cls.execute_soa_method(
            method_name='modifySchedules',
            library='ProjectManagement',
            service_date='2009_10',
            service_name='ScheduleManagement',
            params={'scheduleModifyContainers': scheduleModifyContainers},
            response_cls=ServiceData,
        )

    @classmethod
    def updateTasks(cls, updates: List[GenericAttributesContainer], scheduleUid: List[str]) -> GenericResponseContainer:
        """
        Updates Schedule Tasks. Note: You must use PERSIST for the ServiceType and ScheduleTaskUpdate for the
        operationType inside the GenericAttributesContainers which are passed.
        
        Exceptions:
        >None
        """
        return cls.execute_soa_method(
            method_name='updateTasks',
            library='ProjectManagement',
            service_date='2009_10',
            service_name='ScheduleManagement',
            params={'updates': updates, 'scheduleUid': scheduleUid},
            response_cls=GenericResponseContainer,
        )



from tcsoa.gen.Internal_BusinessModeler._2010_04.services import DataModelManagementService as imp0
from tcsoa.gen.Internal_BusinessModeler._2010_09.services import DataModelManagementService as imp1
from tcsoa.gen.Internal_BusinessModeler._2013_05.services import DynamicLOVQueryService as imp2
from tcsoa.gen.Internal_BusinessModeler._2007_01.services import DataModelManagementService as imp3
from tcsoa.gen.Internal_BusinessModeler._2011_06.services import DataModelManagementService as imp4
from tcsoa.utils import TcService


class DataModelManagementService(TcService):
    deployDataModel = imp0.deployDataModel
    deployDataModel2 = imp1.deployDataModel2
    exportDataModel = imp3.exportDataModel
    getChangedTemplateFiles = imp0.getChangedTemplateFiles
    getSiteTemplateDeployInfo = imp1.getSiteTemplateDeployInfo
    getTemplateFiles = imp0.getTemplateFiles
    importDataModel = imp3.importDataModel
    updateClientMetaCache = imp4.updateClientMetaCache
    updateServerMetaCache = imp4.updateServerMetaCache


class DynamicLOVQueryService(TcService):
    executeDynamicLOVQuery = imp2.executeDynamicLOVQuery



from __future__ import annotations

from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService
from tcsoa.gen.Internal.ActiveWorkspaceBom._2020_05.OccurrenceManagement import UpdateWorkingContextInput, UpdateContentBasedOnRevInput
from tcsoa.gen.BusinessObjects import WorkspaceObject


class OccurrenceManagementService(TcService):

    @classmethod
    def saveWorkingContext(cls, workingContexts: List[WorkspaceObject]) -> ServiceData:
        """
        The operation provides capability to save pending changes to working contexts e.g. Fnd0AppSession or
        Awb0SavedBookmark for Active Workspace. The configuration and recipe are extracted from Awb0AutoBookmark of
        Working Context.
        """
        return cls.execute_soa_method(
            method_name='saveWorkingContext',
            library='Internal-ActiveWorkspaceBom',
            service_date='2020_05',
            service_name='OccurrenceManagement',
            params={'workingContexts': workingContexts},
            response_cls=ServiceData,
        )

    @classmethod
    def updateContentBasedOnRevision(cls, input: UpdateContentBasedOnRevInput) -> ServiceData:
        """
        This operation reconfigures and updates content for impacted occurrence of input WorkspaceObject. As the
        operation is invoked for loaded structure, Awb0Element objects are already created and cached in business
        object registry for opened structure. This operation makes use of BOM event listener mechanism and retrieves
        Awb0Element for corresponding WorkspaceObject from business object registry's cache. The Awb0Element object
        which represents configured revision is returned in ServiceData as updated object. As structure can have
        multiple occurrences configured for input WorkspaceObject, one or more Awb0Element objects may be returned in
        ServiceData.
        """
        return cls.execute_soa_method(
            method_name='updateContentBasedOnRevision',
            library='Internal-ActiveWorkspaceBom',
            service_date='2020_05',
            service_name='OccurrenceManagement',
            params={'input': input},
            response_cls=ServiceData,
        )

    @classmethod
    def updateWorkingContext(cls, input: List[UpdateWorkingContextInput]) -> ServiceData:
        """
        The operation creates and updates the configuration and recipe on a Fnd0StructureContextData object under the
        Fnd0AppSession of Fnd0TempAppSession working context.
        
        Use cases:
        Case 1: User creates Fnd0AppSession from configured BOM Structure
        User configures structure by applying configuration and recipe. The configured structure is used to create new
        Working Context of type Fnd0AppSession. The operation createRelateAndSubmitObjects is called which creates
        Fnd0AppSession object. Upon successful creation of Fnd0AppSession Active Workspace creates and attaches
        Fnd0StructureContextData. The configuration and recipe of source structure is extracted from Awb0AutoBookmark
        and persisted on newly created Session.
        
        Case 2: User performs SaveAs operation on Fnd0AppSession
        User updated configuration and recipe of Fnd0AppSession. The changes are not automatically persisted to
        Fnd0AppSession but updated in Awb0AutoBookmark associated with Fnd0AppSession. Now user performs SaveAs
        operation on opened Fnd0AppSession.
        saveAsObjectsAndRelate uses deep copy rules to create new copy of Fnd0AppSession. Once new copy of
        Fnd0AppSession is created we have to update configuration and recipe on new copy using unsaved changes to
        original Fnd0AppSession on which SaveAs action was performed.
        """
        return cls.execute_soa_method(
            method_name='updateWorkingContext',
            library='Internal-ActiveWorkspaceBom',
            service_date='2020_05',
            service_name='OccurrenceManagement',
            params={'input': input},
            response_cls=ServiceData,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class ApplySignaturesInputData(TcBaseObj):
    """
    This structure contains input parameters required for applyDigitalSignatures operation.    
    
    :var object: Target business object to which Digital Signature is to be applied.
    :var encryptedString: CMS (Cryptographic Message Syntax ) string.
    """
    object: BusinessObject = None
    encryptedString: str = ''


@dataclass
class GetSignatureMessagesOutput(TcBaseObj):
    """
    This structure contains computed signature message.
    
    :var targetObject: Target object for which signature message is computed.
    :var message: Signature message computed for the targetObject.
    """
    targetObject: BusinessObject = None
    message: str = ''


@dataclass
class GetSignatureMessagesResponse(TcBaseObj):
    """
    This structure contains the output parameters of the getSignatureMessages service operation.
    
    :var output: List of getSignatureMessagesOuput structures. 
    :var serviceData: ServiceData Containing list of partial errors and details of business objects for which signature
    message computation was not successful.
    """
    output: List[GetSignatureMessagesOutput] = ()
    serviceData: ServiceData = None


@dataclass
class VoidSignaturesInputData(TcBaseObj):
    """
    This structure contains input parameters required for voidDigitalSignatures operation.
    
    :var targetObject: business object on which digital signatures are to be voided.
    :var signatureobject: List of digital signature objects to be voided.
    :var comment: User comments (optional).
    """
    targetObject: BusinessObject = None
    signatureobject: List[BusinessObject] = ()
    comment: str = ''



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BOMLine
from typing import List
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class ReSequenceParameter(TcBaseObj):
    """
    This structure provides a set of input values for the re-sequence action.
    
    :var bomLines: the selected BOM lines to re-sequence find numbers of their children
    :var startNumber: the start number of the new Find Number
    :var increNumber: the incremental between find numbers
    :var recursive: recursively re-sequence the lines and their children
    """
    bomLines: List[BOMLine] = ()
    startNumber: int = 0
    increNumber: int = 0
    recursive: bool = False



from __future__ import annotations

from typing import List
from tcsoa.utils import TcBaseObj
from tcsoa.gen.Internal.VendorManagementAW._2019_12.VendorManagement import ViewModelObject
from dataclasses import dataclass


@dataclass
class FilterCriteriaInput(TcBaseObj):
    """
    A structure representing the input filter criteria for columns.
    
    :var columnName: Internal name of the property on which column filter is applied.
    :var operation: The type of column filter operation applied. The supported operations are: "CONTAINS", "EQUAL",
    "GREATER THAN", "LESS THAN" and "BETWEEN".
    :var values: A list of values to filter. These are not a fixed set of values like operation.
    The "BETWEEN" operation is used only for Date type. It requires two values and the result will be the rows between
    the range given including min and max value. For example if min date is 2020-03-01T00:00:00+0530 and max date is
    2020-03-05T00:00:00+0530, then the result will be shown from date 2020-03-01T00:00:00+0530 to
    2020-03-05T00:00:00+0530.
    """
    columnName: str = ''
    operation: str = ''
    values: List[str] = ()


@dataclass
class OrderedViewModelInput(TcBaseObj):
    """
    The OrderedViewModelInput structure holds the information about the ViewModelObject structures and the information
    about filter criteria and sort criteria.
    
    :var viewModelRows: A list of view model rows for each &bull; displayed node in a table.
    :var columnFilters: A list of filters to be applied on the columns. The must be internal name of the properties on
    which filtering to be applied along with values of the filter must be sent as input.
    :var sortCriteria: A list of columns and the sort direction. The must be internal name of the property on which
    sorting to be applied along with sort direction.
    """
    viewModelRows: List[ViewModelObject] = ()
    columnFilters: List[FilterCriteriaInput] = ()
    sortCriteria: List[SortCriteria] = ()


@dataclass
class SortCriteria(TcBaseObj):
    """
    Stores column internal name and the direction in which column should be sorted.
    
    :var fieldName: The name of the property on which to perform the sorting.
    :var sortingOrder: The order of sorting of the column. Valid sorting orders are integer values between 1 and
    maximum number of displayed columns. If supplied value is not in this range, then column would not participate in
    sorting.
    :var isAscending: If true, columns will be sorted in ascending direction else it will be sorted in descending
    direction.
    """
    fieldName: str = ''
    sortingOrder: int = 0
    isAscending: bool = False



from __future__ import annotations

from tcsoa.gen.Internal.DocumentManagement._2013_12.Markup import MarkupResponse
from tcsoa.utils import TcService
from tcsoa.gen.BusinessObjects import WorkspaceObject


class MarkupService(TcService):

    @classmethod
    def loadMarkups(cls, baseObject: WorkspaceObject, version: str, message: str) -> MarkupResponse:
        """
        This operation loads the markups visible to the current session user.
        
        Use cases:
        When the user starts Markup tool on the client, the application loads from the server the markups made by other
        users that are visible to this user on the client.
        """
        return cls.execute_soa_method(
            method_name='loadMarkups',
            library='Internal-DocumentManagement',
            service_date='2013_12',
            service_name='Markup',
            params={'baseObject': baseObject, 'version': version, 'message': message},
            response_cls=MarkupResponse,
        )

    @classmethod
    def saveMarkups(cls, baseObject: WorkspaceObject, version: str, message: str, markups: str) -> MarkupResponse:
        """
        This operation saves the markups to the server.
        
        Use cases:
        After the user modified the markups on the client, save them to the server so that other users can read the
        updated markups.
        """
        return cls.execute_soa_method(
            method_name='saveMarkups',
            library='Internal-DocumentManagement',
            service_date='2013_12',
            service_name='Markup',
            params={'baseObject': baseObject, 'version': version, 'message': message, 'markups': markups},
            response_cls=MarkupResponse,
        )



from __future__ import annotations

from tcsoa.gen.Internal.Core._2012_10.DataManagement import CreateIn2, CreateResponse2
from typing import List
from tcsoa.utils import TcService


class DataManagementService(TcService):

    @classmethod
    def createRelateAndSubmitObjects(cls, inputs: List[CreateIn2]) -> CreateResponse2:
        """
        This is a generic operation for creation of business objects. This will also create any secondary (compounded)
        objects that need to be created, assuming the CreateInput2 for the secondary object is represented in the
        recursive CreateInput2 object. e.g. Item is primary object that also creates Item Master and ItemRevision.
        ItemRevision in turn creates ItemRevision Master. The input for all these levels is passed in through the
        recursive CreateInput2 object.
        
        This operation also performs following tasks:
        - Relate the created business object and the additional data passed in through the dataToBeRelated input of
        CreateIn2 object.
        - Submit the created business object to the workflow process. The input for creating the workflow process is
        passed in through the workflowData input of CreateIn2 object.
        - Relate the created business object to the input target object.
        
        
        
        Use cases:
        Use this operation to create an object after obtaining user input on the fields of the create dialog. This call
        is typically preceded by a call to Teamcenter::Soa::Core::_2008_06::PropDescriptor::getCreateDesc or to the
        Client Meta Model layer to retrieve Create Descriptor for a business object.
        
        Create Item
        For example, to create an Item, client will get the Create Descriptor associated with the Item from the client
        Meta model (The associated descriptor type can be found by looking at the constant value for the CreateInput
        constant that is attached to Item). Alternatively, for clients that do not use the client Meta model, the
        Descriptor for Item can be obtained by invoking getCreateDesc operation. The descriptor information can then be
        used to populate the Create dialog for the business object. Once the Create dialog is populated the
        createObjects operation can be called to create the object.
        
        Create Problem Report
        User want to create a new Problem Report (Change) object and attach an existing Word doc dataset as "Reference"
        or "ProblemItem" relation. Here User also wants to submit the created Problem Report object to the workflow
        with predefined Workflow template for Problem Report object type.
        """
        return cls.execute_soa_method(
            method_name='createRelateAndSubmitObjects',
            library='Internal-Core',
            service_date='2012_10',
            service_name='DataManagement',
            params={'inputs': inputs},
            response_cls=CreateResponse2,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class ExportToApplicationInputData2(TcBaseObj):
    """
    The ExportToApplicationInput2 represents the data required to export selected objects to MSWord or MSExcel.
    
    :var objectsToExport: The list of Teamcenter business objects to be exported to MSWord or MSExcel.
    :var targetObjectsToExport: The list of Teamcenter business objects to be exported which acts as target objects
    used for comparing the contents. This is an optional parameter and can be empty.
    :var attributesToExport: The list of attributes to export. This is an optional parameter which can be empty.
    :var applicationFormat: The application formats supported are :
    "MSExcel","MSWordXMLLive","MExcelLive","MSExcelReimport","MSWordCompare","HTML"
    :var templateName: The name of the MSWord or MSExcel template.
    """
    objectsToExport: List[BusinessObject] = ()
    targetObjectsToExport: List[BusinessObject] = ()
    attributesToExport: List[str] = ()
    applicationFormat: str = ''
    templateName: str = ''


@dataclass
class ExportToApplicationResponse2(TcBaseObj):
    """
    This structure contains the html contents of the input SpecElement objects or a file ticket of .docm or .xlsm file.
    
    :var transientFileReadTickets: The transient file read tickets for the exported file.
    :var contents: The HTML contents associated to the objects sent as input to the service operation. This is an
    optional parameter in the response . The parameter targetObjectsToExport in ExportToApplicationInput2 is not used
    to output the HTML contents.
    :var serviceData: The ServiceData. If an error occurs then ServiceData will contain partial error such as error
    generating transient file read ticket due to a configuration issue at the server.
    """
    transientFileReadTickets: List[str] = ()
    contents: List[str] = ()
    serviceData: ServiceData = None
